import matplotlib
matplotlib.rc('xtick', labelsize=14)
matplotlib.rc('ytick', labelsize=14)
matplotlib.rc('axes', labelsize=18)
legendfontsize = 11

import matplotlib.pyplot as plt
import numpy as np
from NuRadioReco.utilities import units
from NuRadioMC.utilities import fluxes
import os

energyBinsPerDecade = 1.
plotUnitsEnergy = units.GeV
plotUnitsFlux = units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1
DIFFUSE = True

# Unless you would like to work with the layout or the models/data from other experiments,
# you don't need to change anything below here
# --------------------------------------------------------------------
# Other planned experiments

# GRAND white paper,
# numerical values, Bustamante
GRAND_energy = np.array(([48192296.5, 67644231.1, 94947581.6, 133271428.0, 187063990.0, 262568931.0, 368550053.0, 517308507.0, 726110577.0, 1019191760.0, 1430569790.0, 2007992980.0, 2818482440.0, 3956111070.0, 5552922590.0, 7794257720.0, 10940266600.0, 15356104100.0, 21554313200.0, 30254315500.0, 42465913900.0,
                          59606499400.0, 83665567300.0, 117435636000.0, 164836371000.0, 231369543000.0, 324757606000.0, 455840043000.0, 639831498000.0, 898087721000.0, 1260584320000.0, 1769396010000.0, 2483580190000.0, 3486031680000.0, 4893104280000.0, 6868115880000.0, 9640304610000.0, 13531436400000.0, 18993151900000.0, 26659388600000.0]))

GRAND_energy *= units.GeV

GRAND_10k = np.array(([8.41513361e-08, 7.38147706e-08, 5.69225180e-08, 3.46647934e-08,
                       1.95651137e-08, 1.40651565e-08, 1.25782087e-08, 1.24621707e-08,
                       1.31123151e-08, 1.45812119e-08, 1.65528260e-08, 1.91930521e-08,
                       2.31554429e-08, 2.87477813e-08, 3.55164030e-08, 4.42563884e-08,
                       5.63965197e-08, 7.45183330e-08, 1.01159657e-07, 1.39040439e-07,
                       1.98526677e-07, 2.61742251e-07, 3.40870828e-07, 4.82745531e-07,
                       6.55876763e-07, 9.07706655e-07, 1.67125879e-06, 1.76142511e-05,
                       2.55022320e-04, 1.88371074e-03, 6.71431813e-03, 1.14286198e-02,
                       1.14294614e-02, 1.72447830e-02, 7.48579143e-02, 3.31883351e-01,
                       8.57786094e-01, 1.24824516e+00, 1.42294586e+00, 1.80135089e+00]))

GRAND_10k *= (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)
#GRAND_10k /= 2 #halfdecade bins
GRAND_10k *= energyBinsPerDecade

GRAND_200k = np.array(([4.26219753e-09, 3.58147708e-09, 2.75670137e-09, 1.85254042e-09,
                        1.13825106e-09, 7.70141315e-10, 6.51758930e-10, 6.35878242e-10,
                        6.69261628e-10, 7.37439217e-10, 8.38784832e-10, 9.81688683e-10,
                        1.18493794e-09, 1.45699379e-09, 1.80867621e-09, 2.26948852e-09,
                        2.91952068e-09, 3.86790849e-09, 5.24530715e-09, 7.31211288e-09,
                        9.98848945e-09, 1.33523293e-08, 1.80893102e-08, 2.46582187e-08,
                        3.41054825e-08, 5.39140368e-08, 3.36553610e-07, 4.57179717e-06,
                        3.59391218e-05, 1.47550853e-04, 3.33777479e-04, 4.92873322e-04,
                        6.68381070e-04, 1.72553598e-03, 7.06643413e-03, 2.10754560e-02,
                        4.06319101e-02, 5.88162853e-02, 7.45423652e-02, 8.83700084e-02]))

GRAND_200k *= (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)
#GRAND_200k /= 2 #halfdecade bins
GRAND_200k *= energyBinsPerDecade

# RADAR proposed from https://arxiv.org/pdf/1710.02883.pdf

Radar = np.array(([
    (1.525e+01, 6.870e-09, 3.430e-07),
    (1.575e+01, 9.797e-10, 3.113e-08),
    (1.625e+01, 4.728e-09, 1.928e-07),
    (1.675e+01, 6.359e-09, 3.706e-07),
    (1.725e+01, 9.128e-09, 8.517e-07),
    (1.775e+01, 1.619e-08, 1.835e-06),
    (1.825e+01, 2.995e-08, 2.766e-06),
    (1.875e+01, 5.562e-08, 8.253e-06),
    (1.925e+01, 1.072e-07, 1.849e-05)]))

Radar[:, 0] = 10**Radar[:, 0] * units.eV
Radar[:, 1] *= (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)
Radar[:, 1] /= 2 #halfdecade bins
Radar[:, 1] *= energyBinsPerDecade
Radar[:, 2] *= (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)
Radar[:, 2] /= 2 #halfdecade bins
Radar[:, 2] *= energyBinsPerDecade
# --------------------------------------------------------------------
# Published data and limits

# IceCube
# log (E^2 * Phi [GeV cm^02 s^-1 sr^-1]) : log (E [Gev])
# Phys Rev D 98 062003 (2018)
# Numbers private correspondence Shigeru Yoshida
ice_cube_limit = np.array(([
    (6.199999125, -7.698484687),
    (6.299999496, -8.162876678),
    (6.400000617, -8.11395291),
    (6.500000321, -8.063634144),
    (6.599999814, -8.004841781),
    (6.699999798, -7.944960162),
    (6.799999763, -7.924197388),
    (6.899999872, -7.899315263),
    (7.299999496, -7.730561153),
    (7.699999798, -7.670680637),
    (8.100001583, -7.683379711),
    (8.500000321, -7.748746801),
    (8.899999872, -7.703060304),
    (9.299999496, -7.512907553),
    (9.699999798, -7.370926525),
    (10.10000158, -7.134626026),
    (10.50000032, -6.926516638),
    (10.89999987, -6.576523031)
]))

ice_cube_limit[:, 0] = 10**ice_cube_limit[:, 0] * units.GeV
ice_cube_limit[:, 1] = 10**ice_cube_limit[:, 1] * (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)
ice_cube_limit[:, 1] *= energyBinsPerDecade

# Fig. 2 from PoS ICRC2017 (2018) 981
# IceCube preliminary
# E (GeV); E^2 dN/dE (GeV cm^-2 s-1 sr-1); yerror down; yerror up

# HESE 6 years
# ice_cube_hese = np.array(([
#
#     (6.526e+04,  2.248e-08,  9.96e-9,  1.123e-8),
#     (1.409e+05,  2.692e-08,  5.91e-9,  7.56e-9),
#     (3.041e+05,  7.631e-09,  3.746e-9, 4.61e-9),
#     (6.644e+05,  2.022e-09,  7.03e-10, 0.),
#     (1.434e+06,  5.205e-09,  3.183e-9,  4.57e-9),
#     (3.096e+06,  4.347e-09,  3.142e-9,  5.428e-9),
#     (6.684e+06,  1.544e-09,  5.37e-10, 0.),
#     (1.46e+07,  4.063e-09,   1.353e-9, 0.),
#     (3.153e+07,  6.093e-09,  2.03e-9,  0.),
#     (6.806e+07,  1.046e-08,  3.641e-9, 0.)
# ]))
# ice_cube_hese[:, 0] = ice_cube_hese[:, 0] * units.GeV
# ice_cube_hese[:, 1] = ice_cube_hese[:, 1] * (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)
# ice_cube_hese[:, 1] *= 3
# ice_cube_hese[:, 2] = ice_cube_hese[:, 2] * (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)
# ice_cube_hese[:, 2] *=  3
# ice_cube_hese[:, 3] = ice_cube_hese[:, 3] * (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)
# ice_cube_hese[:, 3] *= 3


# HESE 8 years
#log(E (GeV)); log(E^2 dN/dE (GeV cm^-2 s-1 sr-1)); format x y -dy +dy
ice_cube_hese = np.array(([
(4.78516,	-7.63256,		0.223256,	0.167442),
(5.10938,	-7.66977,		0.139535,	0.102326),
(5.42969,	-8.36744,		0.930233,	0.297674),
(5.75391,	-8.51628,		0.2,       	0.	),
(6.07813,	-8.38605,		0.604651,	0.288372),
(6.39844,	-8.35814,		0.455814,	0.334884),
(6.72266,	-9.0,    		0.2 ,    	0)
]))

# get uncertainties in right order
ice_cube_hese[:, 2] = 10**ice_cube_hese[:, 1] - 10**(ice_cube_hese[:, 1]-ice_cube_hese[:, 2])
ice_cube_hese[:, 3] = 10**(ice_cube_hese[:, 1]+ice_cube_hese[:, 3]) - 10**ice_cube_hese[:, 1]


ice_cube_hese[:, 0] = 10**ice_cube_hese[:, 0] * units.GeV
ice_cube_hese[:, 1] = 10**ice_cube_hese[:, 1] * (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)
ice_cube_hese[:, 1] *= 3

ice_cube_hese[:, 2] *= (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)
ice_cube_hese[:, 2] *=  3

ice_cube_hese[:, 3] *= (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)
ice_cube_hese[:, 3] *= 3




# Ice cube


def ice_cube_nu_fit(energy, slope=-2.19, offset=1.01):
    # ApJ slope=-2.13, offset=0.9
    flux = 3 * offset * (energy / (100 * units.TeV))**slope * 1e-18 * \
        (units.GeV**-1 * units.cm**-2 * units.second**-1 * units.sr**-1)
    return flux


def get_ice_cube_mu_range():
    energy = np.arange(1e5, 5e6, 1e5) * units.GeV
#     upper = np.maximum(ice_cube_nu_fit(energy, offset=0.9, slope=-2.), ice_cube_nu_fit(energy, offset=1.2, slope=-2.13)) # APJ
    upper = np.maximum(ice_cube_nu_fit(energy, offset=1.01, slope=-2.09),
                    ice_cube_nu_fit(energy, offset=1.27, slope=-2.19),ice_cube_nu_fit(energy, offset=1.27, slope=-2.09)) # ICRC
    upper *= energy**2
#     lower = np.minimum(ice_cube_nu_fit(energy, offset=0.9, slope=-2.26),
#                        ice_cube_nu_fit(energy, offset=0.63, slope=-2.13)) #ApJ
    lower = np.minimum(ice_cube_nu_fit(energy, offset=1.01, slope=-2.29),
                       ice_cube_nu_fit(energy, offset=0.78, slope=-2.19)) #ICRC
    lower *= energy**2
    return energy, upper, lower


def get_ice_cube_hese_range():
    energy = np.arange(1e5, 5e6, 1e5) * units.GeV
    upper = np.maximum(ice_cube_nu_fit(energy, offset=2.46, slope=-2.63),
                       ice_cube_nu_fit(energy, offset=2.76, slope=-2.92))
    upper *= energy**2
    lower = np.minimum(ice_cube_nu_fit(energy, offset=2.46, slope=-3.25),
                       ice_cube_nu_fit(energy, offset=2.16, slope=-2.92))
    lower *= energy**2
    return energy, upper, lower


# ANITA I - III
# Phys. Rev. D 98, 022001 (2018)
anita_limit = np.array(([
    (9.94e17,	3.79e-14 * 9.94e17 / 1e9),
    (2.37e18,	2.15e-15 * 2.37e18 / 1e9),
    (5.19e18,	2.33e-16 * 5.19e18 / 1e9),
    (1.10e19,	3.64e-17 * 1.10e19 / 1e9),
    (3.55e19,	4.45e-18 * 3.55e19 / 1e9),
    (1.11e20,	9.22e-19 * 1.11e20 / 1e9),
    (4.18e20,	2.97e-19 * 4.18e20 / 1e9),
    (9.70e20,	1.62e-19 * 9.70e20 / 1e9)
]))

anita_limit[:, 0] *= units.eV
anita_limit[:, 1] *= (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)
anita_limit[:, 1] /= 2
anita_limit[:, 1] *= energyBinsPerDecade


# Auger neutrino limit
# Auger 9 years, all flavour (x3)
auger_limit = np.array(([
    (16.7523809524, 4.462265901e-07),
    (17.2523809524, 1.103901153e-07),
    (17.7523809524, 6.487559078e-08),
    (18.2380952381, 7.739545498e-08),
    (18.7523809524, 1.387743075e-07),
    (19.2571428571, 3.083827665e-07),
    (19.7523809524, 7.467202523e-07),
    (20.2476190476, 1.998499395e-06)
]))
auger_limit[:, 0] = 10 ** auger_limit[:, 0] * units.eV
auger_limit[:, 1] *= (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)
auger_limit[:, 1] /= 2  # half-decade binning
auger_limit[:, 1] *= energyBinsPerDecade

# ARA Published 2sta x 1yr analysis level limit:
ara_1year = np.array((
[9.80923e+15, 3.11547e+16, 9.79329e+16, 3.1057e+17, 9.75635e+17,
3.0924e+18, 9.80103e+18, 3.07732e+19, 9.75028e+19],
[0.000166106, 1.24595e-05, 4.06271e-06, 2.04351e-06, 1.48811e-06,
1.42649e-06, 1.50393e-06, 2.10936e-06, 3.25384e-06]
))

ara_1year = ara_1year.T

ara_1year[:,0] *= units.eV
ara_1year[:,1] *= (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)
ara_1year[:,1] /= 2 #half-decade binning
ara_1year[:,1] *= energyBinsPerDecade

# Ongoing analysis 2sta x 4yr *trigger* level projected limit (analysis
# level not fully defined yet)

ara_4year = np.array((

[9.91245E+15,3.11814E+16,9.80021E+16,3.10745E+17,9.94099E+17,3.0936E+18,9.71449E+18,3.07805E+19,9.75192E+19],
# [1.01518E+16,1.01357E+17,1.01748E+18,1.0234E+19,1.03113E+20],
# [7.27626e-06,5.06909e-07,2.13658e-07,2.02468e-07,4.46012e-07]
[5.35394e-06,1.24309e-06,4.20315e-07,2.24199e-07,1.61582e-07,1.50329e-07,1.63715e-07,2.24543e-07,3.36398e-07]
))

ara_4year = ara_4year.T

ara_4year[:,0] *= units.eV
ara_4year[:,1] *= (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)
ara_4year[:,1] /= 2 #half-decade binning
ara_4year[:,1] *= energyBinsPerDecade



# get 10% proton flux
def get_proton_10(energy):
    vanVliet_reas = np.loadtxt(os.path.join(os.path.dirname(__file__), "ReasonableNeutrinos1.txt"))
    E = vanVliet_reas[0, :] * plotUnitsEnergy
    f = vanVliet_reas[1, :] * plotUnitsFlux / E**2
    from scipy.interpolate import interp1d
    getE = interp1d(E, f)
    return getE(energy)



def get_E2_limit_figure(diffuse = True,
                        show_ice_cube_EHE_limit=True,
                        show_ice_cube_HESE_data=True,
                        show_ice_cube_HESE_fit=True,
                        show_ice_cube_mu=True,
                        show_anita_I_III_limit=True,
                        show_auger_limit=True,
                        show_ara=True,
                        show_neutrino_best_fit=True,
                        show_neutrino_best_case=True,
                        show_neutrino_worst_case=True,
                        show_grand_10k=True,
                        show_grand_200k=False,
                        show_radar=False):

    # Limit E2 Plot
    # ---------------------------------------------------------------------------
    fig, ax = plt.subplots(1, 1, figsize=(7, 6))

    # Neutrino Models
    # Version for a diffuse flux and for a source dominated flux
    if diffuse:
        Heinze_band = np.loadtxt(os.path.join(os.path.dirname(__file__), "talys_neu_bands.out"))
        best_fit, = ax.plot(Heinze_band[:, 0], Heinze_band[:, 1] * Heinze_band[:, 0]**2, c='k',
                            label=r'Best fit UHECR, Heinze et al.', linestyle='-.')

        Heinze_evo = np.loadtxt(os.path.join(os.path.dirname(__file__), "talys_neu_evolutions.out"))
        best_fit_3s, = ax.plot(Heinze_evo[:, 0]*units.GeV/plotUnitsEnergy, Heinze_evo[:, 6] * Heinze_evo[:, 0] **
                        2, color='0.5',label=r'Best fit UHECR + 3$\sigma$, Heinze et al.', linestyle='-.')

        vanVliet_max_1 = np.loadtxt(os.path.join(os.path.dirname(__file__), "MaxNeutrinos1.txt"))
        vanVliet_max_2 = np.loadtxt(os.path.join(os.path.dirname(__file__), "MaxNeutrinos2.txt"))
        vanVliet_reas = np.loadtxt(os.path.join(os.path.dirname(__file__), "ReasonableNeutrinos1.txt"))

        vanVliet_max = np.maximum(vanVliet_max_1[1, :], vanVliet_max_2[1, :])

        prot10, = ax.plot(vanVliet_reas[0, :]*units.GeV/plotUnitsEnergy, vanVliet_reas[1, :],
                          label=r'10% protons in UHECRs, van Vliet et al.', linestyle='--', color='k')

        prot = ax.fill_between(vanVliet_max_1[0,:]*units.GeV/plotUnitsEnergy,vanVliet_max,vanVliet_reas[1,:]/50,color='0.9',label=r'allowed from UHECRs, van Vliet et al.')

        first_legend = plt.legend(handles=[best_fit, best_fit_3s, prot10, prot], loc=4,fontsize=legendfontsize, handlelength=4)

        plt.gca().add_artist(first_legend)
    else:
        tde = np.loadtxt(os.path.join(os.path.dirname(__file__),"TDEneutrinos.txt"))
        ll_grb = np.loadtxt(os.path.join(os.path.dirname(__file__),"LLGRBneutrinos.txt"))
        pulsars = np.loadtxt(os.path.join(os.path.dirname(__file__),"Pulsar_Fang+_2014.txt"))
        clusters = np.loadtxt(os.path.join(os.path.dirname(__file__),"cluster_Fang_Murase_2018.txt"))

        # Fang & Metzger
        data_ns_merger = np.array((
        [164178.9149064658, 6.801708660714134e-11],
        [336720.74740929523, 1.4132356758632395e-10],
        [835305.4165187279, 3.649486484772094e-10],
        [2160958.1870687287, 9.239856704429993e-10],
        [8002898.345863899, 3.085108101843864e-9],
        [20681309.183352273, 6.161686112812425e-9],
        [61887155.98482043, 1.0930162266889215e-8],
        [132044526.30261868, 1.3058095134564553e-8],
        [253159530.43095005, 1.0506528126116853e-8],
        [436411840.74101496, 6.5380862245683814e-9],
        [635712891.7663972, 3.910881690144994e-9],
        [944515747.1734984, 1.773891442500038e-9],
        [1211896737.260516, 9.059026812485584e-10],
        [1586152410.074502, 3.578063705414756e-10],
        [1948716060.6917415, 1.358461111073226e-10],
        [2344547243.608689, 5.26053655617631e-11]))

        data_ns_merger[:,0] *= units.GeV
        data_ns_merger[:,1] *= units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1

        ns_merger, = ax.plot(data_ns_merger[:,0]/plotUnitsEnergy,data_ns_merger[:,1]/plotUnitsFlux,color='palevioletred',label='NS-NS merger, Fang & Metzger 1707.04263',linestyle=(0, (3, 5, 1, 5)))

        ax.fill_between(tde[:,0]*units.GeV/plotUnitsEnergy,tde[:,2]*3,tde[:,3]*3,color='thistle',alpha=0.5)
        p_tde, = ax.plot(tde[:,0]*units.GeV/plotUnitsEnergy,tde[:,1]*3,label="TDE, Biehl et al. (1711.03555)", color='darkmagenta', linestyle=':',zorder=1)

        ax.fill_between(ll_grb[:,0]*units.GeV/plotUnitsEnergy,ll_grb[:,2]*3,ll_grb[:,3]*3, color='0.8')
        p_ll_grb, = ax.plot(ll_grb[:,0]*units.GeV/plotUnitsEnergy,ll_grb[:,1]*3,label="LLGRB, Boncioli et al. (1808.07481)",linestyle='-.',c='k',zorder=1)

        p_pulsar = ax.fill_between(pulsars[:,0]*units.GeV/plotUnitsEnergy,pulsars[:,1],pulsars[:,2],label="Pulsar, Fang et al. (1311.2044)",color='wheat',alpha=0.5)
        p_cluster, = ax.plot(clusters[:,0]*units.GeV/plotUnitsEnergy,clusters[:,1],label="Clusters, Fang & Murase, (1704.00015)",color="olive",zorder=1,linestyle=(0, (5, 10)))

        first_legend = plt.legend(handles=[p_tde, p_ll_grb, p_pulsar,p_cluster,ns_merger], loc=3,fontsize=legendfontsize, handlelength=4)

        plt.gca().add_artist(first_legend)

    #-----------------------------------------------------------------------

    if show_grand_10k:
        ax.plot(GRAND_energy / plotUnitsEnergy, GRAND_10k / plotUnitsFlux, linestyle=":", color='saddlebrown')
        if energyBinsPerDecade == 2:
            ax.annotate('GRAND 10k',
                            xy=(1e10, 1.1e-7), xycoords='data',
                            horizontalalignment='left', color='saddlebrown', rotation=40,fontsize=legendfontsize)
        else:
            ax.annotate('GRAND 10k',
                xy=(5e9, 7e-8), xycoords='data',
                horizontalalignment='left', color='saddlebrown', rotation=40,fontsize=legendfontsize)



    if show_grand_200k:
        ax.plot(GRAND_energy / plotUnitsEnergy, GRAND_200k / plotUnitsFlux, linestyle=":", color='saddlebrown')
        ax.annotate('GRAND 200k',
                    xy=(1e10, 6e-9), xycoords='data',
                    horizontalalignment='left', color='saddlebrown', rotation=40,fontsize=legendfontsize)
    if show_radar:
        ax.fill_between(Radar[:, 0] / plotUnitsEnergy, Radar[:, 1] / plotUnitsFlux,
                        Radar[:, 2] / plotUnitsFlux, facecolor='None', hatch='x', edgecolor='0.8')
        ax.annotate('Radar',
                    xy=(1e9, 4.5e-8), xycoords='data',
                    horizontalalignment='left', color='0.7', rotation=45,fontsize=legendfontsize)

    if show_ice_cube_EHE_limit:
        ax.plot(ice_cube_limit[:, 0] / plotUnitsEnergy, ice_cube_limit[:, 1] / plotUnitsFlux, color='dodgerblue')
        if energyBinsPerDecade == 2:
            ax.annotate('IceCube',
                    xy=(0.7e7, 4e-8), xycoords='data',
                    horizontalalignment='center', color='dodgerblue', rotation=0,fontsize=legendfontsize)
        else:
            ax.annotate('IceCube',
                    xy=(4e6, 4e-8), xycoords='data',
                    horizontalalignment='center', color='dodgerblue', rotation=0,fontsize=legendfontsize)

    if show_ice_cube_HESE_data:
        # data points
        uplimit = np.copy(ice_cube_hese[:, 3])
        uplimit[np.where(ice_cube_hese[:, 3] == 0)] = 1
        uplimit[np.where(ice_cube_hese[:, 3] != 0.)] = 0


        ax.errorbar(ice_cube_hese[:, 0] / plotUnitsEnergy, ice_cube_hese[:, 1] / plotUnitsFlux, yerr=ice_cube_hese[:,2:].T / plotUnitsFlux, uplims=uplimit, color='dodgerblue', marker='o', ecolor='dodgerblue', linestyle='None',zorder=3)


    if show_ice_cube_HESE_fit:
        ice_cube_hese_range = get_ice_cube_hese_range()
        ax.fill_between(ice_cube_hese_range[0] / plotUnitsEnergy, ice_cube_hese_range[1] / plotUnitsFlux,
                        ice_cube_hese_range[2] / plotUnitsFlux, hatch='//', edgecolor='dodgerblue', facecolor='azure',zorder=2)
        plt.plot(ice_cube_hese_range[0] / plotUnitsEnergy, ice_cube_nu_fit(ice_cube_hese_range[0],
                                                                           offset=2.46, slope=-2.92) * ice_cube_hese_range[0]**2 / plotUnitsFlux, color='dodgerblue')

    if show_ice_cube_mu:
        # mu fit
        ice_cube_mu_range = get_ice_cube_mu_range()
        ax.fill_between(ice_cube_mu_range[0] / plotUnitsEnergy, ice_cube_mu_range[1] / plotUnitsFlux,
                        ice_cube_mu_range[2] / plotUnitsFlux, hatch='\\', edgecolor='dodgerblue', facecolor='azure',zorder=2)
        plt.plot(ice_cube_mu_range[0] / plotUnitsEnergy,
                 ice_cube_nu_fit(ice_cube_mu_range[0], offset=1.01, slope=-2.19) * ice_cube_mu_range[0]**2 / plotUnitsFlux,
                 color='dodgerblue')

        # Extrapolation
        energy_placeholder =   np.array(([1e14,1e17]))*units.eV
        plt.plot(energy_placeholder / plotUnitsEnergy,
                 ice_cube_nu_fit(energy_placeholder, offset=1.01, slope=-2.19) * energy_placeholder**2 / plotUnitsFlux,
                 color='dodgerblue',linestyle=':')


    if show_anita_I_III_limit:
        ax.plot(anita_limit[:, 0] / plotUnitsEnergy, anita_limit[:, 1] / plotUnitsFlux, color='darkorange')
        if energyBinsPerDecade == 2:
            ax.annotate('ANITA I - III',
                        xy=(7e9, 1e-6), xycoords='data',
                        horizontalalignment='left', color='darkorange',fontsize=legendfontsize)
        else:
            ax.annotate('ANITA I - III',
                        xy=(7e9, 5e-7), xycoords='data',
                        horizontalalignment='left', color='darkorange',fontsize=legendfontsize)

    if show_auger_limit:
        ax.plot(auger_limit[:, 0] / plotUnitsEnergy, auger_limit[:, 1] / plotUnitsFlux, color='forestgreen')
        if energyBinsPerDecade == 2:
            ax.annotate('Auger',
                        xy=(1.1e8, 2.1e-7), xycoords='data',
                        horizontalalignment='left', color='forestgreen', rotation=0,fontsize=legendfontsize)
        else:
            ax.annotate('Auger',
                        xy=(2e8, 6e-8), xycoords='data',
                        horizontalalignment='left', color='forestgreen', rotation=0,fontsize=legendfontsize)

    if show_ara:
        ax.plot(ara_1year[:,0]/plotUnitsEnergy,ara_1year[:,1]/ plotUnitsFlux,color='indigo')
#         ax.plot(ara_4year[:,0]/plotUnitsEnergy,ara_4year[:,1]/ plotUnitsFlux,color='indigo',linestyle='--')
        if energyBinsPerDecade == 2:
            ax.annotate('ARA',
                        xy=(5e8, 6e-7), xycoords='data',
                        horizontalalignment='left', color='indigo', rotation=0,fontsize=legendfontsize)
        else:
            ax.annotate('ARA',
                    xy=(1e10, 1.05e-6), xycoords='data',
                    horizontalalignment='left', color='indigo', rotation=0,fontsize=legendfontsize)

    ax.set_yscale('log')
    ax.set_xscale('log')

    ax.set_xlabel(r'neutrino energy [GeV]')
    ax.set_ylabel(r'$E^2\Phi$ [GeV cm$^{-2}$ s$^{-1}$ sr$^{-1}$]')

    if diffuse:
        ax.set_ylim(1e-11, 2e-6)
        ax.set_xlim(1e5, 1e11)
    else:
        ax.set_ylim(1e-11, 2e-6)
        ax.set_xlim(1e5, 1e11)

    plt.tight_layout()
    return fig, ax

def add_limit(ax, limit_labels, E, Veff, n_stations, label, livetime=3*units.year, linestyle='-',color='r',linewidth=3,band=False):
    E = np.array(E)
    Veff = np.array(Veff)

    if band:

        limit_lower = fluxes.get_limit_e2_flux(energy = E,
                                         veff = Veff[0],
                                         livetime = livetime,
                                         signalEff = n_stations,
                                         energyBinsPerDecade=energyBinsPerDecade,
                                         upperLimOnEvents=2.44,
                                         nuCrsScn='ctw')
        limit_upper = fluxes.get_limit_e2_flux(energy = E,
                                         veff = Veff[1],
                                         livetime = livetime,
                                         signalEff = n_stations,
                                         energyBinsPerDecade=energyBinsPerDecade,
                                         upperLimOnEvents=2.44,
                                         nuCrsScn='ctw')

        plt1 = ax.fill_between(E/plotUnitsEnergy,limit_upper/ plotUnitsFlux,
        limit_lower/plotUnitsFlux,color=color,alpha=0.2)

    else:

        limit = fluxes.get_limit_e2_flux(energy = E,
                                         veff = Veff,
                                         livetime = livetime,
                                         signalEff = n_stations,
                                         energyBinsPerDecade=energyBinsPerDecade,
                                         upperLimOnEvents=2.44,
                                         nuCrsScn='ctw')

        print(E/plotUnitsEnergy,limit/ plotUnitsFlux)

    #         _plt, = ax.plot(E/plotUnitsEnergy,limit/ plotUnitsFlux, linestyle=linestyle, color=color,
    #                         label="{2}: {0} stations, {1} years".format(n_stations,int(livetime/units.year),label),
    #                         linewidth=linewidth)
        _plt, = ax.plot(E/plotUnitsEnergy,limit/ plotUnitsFlux, linestyle=linestyle, color=color,
                        label="{1}: {0} years".format(int(livetime/units.year),label),
                        linewidth=linewidth)

        limit_labels.append(_plt)


    return limit_labels




if __name__=="__main__":
    # 50 meter
    veff = np.array((
    [1e+16, 3.162277660168379e+16, 1e+17, 3.1622776601683795e+17, 1e+18, 3.1622776601683794e+18, 1e+19, 3.162277660168379e+19],
    [0.007467602898257461, 0.06986834193426224, 0.5333379226865426, 2.1410489793474383, 5.896654567671568, 11.343574036186226, 18.415350759353128, 27.81614390854279]
    )).T

    veff[:,0] *= units.eV
    veff[:,1] *= units.km**3 * units.sr
    veff_label = 'One current design'

#     strawman_veff_pa = np.array(( [1.00000000e+16, 3.16227766e+16, 1.00000000e+17, 3.16227766e+17, 1.00000000e+18, 3.16227766e+18, 1.00000000e+19, 3.16227766e+19],
#                               [1.82805666e+07, 1.34497197e+08, 6.32044851e+08, 2.20387046e+09, 4.86050340e+09, 8.18585201e+09, 1.25636305e+10, 1.83360237e+10])).T
#
#     strawman_veff_pa[:,0] *= units.eV
#     strawman_veff_pa[:,1] *= units.m**3 * units.sr

#     strawman_pa_label = 'Strawman + PA@15m@2s'
#     strawman_pa_label = 'One current design'
    fig, ax = get_E2_limit_figure(diffuse=DIFFUSE)
    labels =[]
    labels = add_limit(ax, labels, veff[:,0],veff[:,1], n_stations = 100, livetime=5*units.year, label= veff_label)
    labels = add_limit(ax, labels, veff[:,0],veff[:,1], n_stations = 1000, livetime=5*units.year, label= veff_label)
    plt.legend(handles=labels, loc=2)
    if DIFFUSE:
        name_plot = "Limit_diffuse.pdf"
    else:
        name_plot = "Limit_sources.pdf"
    plt.savefig(name_plot)
    plt.show()



#!/usr/bin/env python
from __future__ import print_function
"""
Merge hdf5 files
"""
import os
from NuRadioMC.utilities.hdf5_manipulator import hdf5
import numpy as np
from collections import OrderedDict
from NuRadioMC.utilities.hdf5_manipulator import msg
from NuRadioMC.utilities.hdf5_manipulator import check


def get_filelist(bases):

    """look for files which match given bases and return them as list

    Keyword arguments:
    bases -- list of 'path/basename'
    """

    filelist = []

    for base in bases:
        path, fname = os.path.dirname(base) or '.', os.path.basename(base)
        filelist.extend([path + '/' + f for f in os.listdir(path)
                        if f.startswith(fname) and f.endswith(".hdf5")])

    return sorted(filelist)


def merge_data(data_list, attrs_list, groups_list, attrs_group_list):

    """Merge dictionaries with data.

    Keyword arguments:
    data_list -- the dictionary with data dictionaries
    """

    data = None
    attrs = None
    groups = None
    attrs_group = None

    for f in data_list:
        size = check.get_size(data_list[f])
        if not data:
            print("\nThe following datasets were found in %s:\n" % f)
            msg.list_dataset(data_list[f])
            data = data_list[f]
            attrs = attrs_list[f]
            groups = groups_list[f]
            attrs_group = attrs_group_list[f]
        else:
            print("\nAdding %(n)d entries from %(f)s" % {"n": size, "f": f})

            if len(attrs['trigger_names']) == 0 and 'trigger_names' in attrs_list[f]:
                attrs['trigger_names'] = attrs_list[f]['trigger_names']
            if len(data_list[f]['triggered']) == 0:
                attrs['n_events'] += attrs_list[f]['n_events']
                continue
            check.check_keys(data, data_list[f])
            check.check_shapes(data, data_list[f])
            for key in data_list[f]:
                data[key] = np.append(data[key], data_list[f][key], axis=0)
            for key in groups_list[f]:
                for key2 in groups_list[f][key]:
                    groups[key][key2] = np.append(groups[key][key2], groups_list[f][key][key2], axis=0)
                    
            attrs['n_events'] += attrs_list[f]['n_events']

    if 'trigger_names' not in attrs:
        attrs['trigger_names'] = []

    return data, attrs, groups, attrs_group


def merge_data_filenames(filelist, outputfile):
    print("The following input files were found:\n")

    for f in filelist:
        print("\t - %s" % f)

    data = OrderedDict()
    attrs = OrderedDict()
    groups = OrderedDict()
    group_attrs = OrderedDict()

    for f in filelist:
        print(f)
        data[f], attrs[f], groups[f], group_attrs[f] = hdf5.load(f)

    hdf5.save(outputfile, *merge_data(data, attrs, groups, group_attrs))

    msg.info("Done")



import numpy as np
from NuRadioMC.utilities import units
import os
from scipy import interpolate as intp
import glob
import pickle
import sys
from matplotlib import pyplot as plt

if __name__ == "__main__":
    library_path = sys.argv[1]
    with open(library_path, 'rb') as fin:
        lib = pickle.load(fin)
        
        f, ax = plt.subplots(1, len(lib), sharey=True)
        
        for iS, shower_type in enumerate(lib):
            b = []
            nb = []
            for E in lib[shower_type]:
                nE = len(lib[shower_type][E]['charge_excess'])
                b.append(E)
                nb.append(nE)

            ax[iS].bar(np.log10(b), nE, linewidth=1, edgecolor='k', width=0.05)
            ax[iS].set_xlabel("log10(shower energy [eV])")
            ax[iS].set_title(shower_type)
        ax[0].set_ylabel('number of showers')
        f.tight_layout()
        plt.show()
                
        


from __future__ import absolute_import, division, print_function
import numpy as np
from radiotools import helper as hp
from radiotools import plthelpers as php
from matplotlib import pyplot as plt
from NuRadioMC.utilities import units
from NuRadioMC.utilities import medium
import h5py
import argparse
import json
import time
import os

parser = argparse.ArgumentParser(description='Plot NuRadioMC event list output.')
parser.add_argument('inputfilename', type=str, nargs = '+', help='path to NuRadioMC hdf5 simulation output')
# parser.add_argument('outputfilename', type=str,
#                     help='name of output file storing the electric field traces at detector positions')
args = parser.parse_args()

filename = os.path.splitext(os.path.basename(args.inputfilename[0]))[0]
dirname = os.path.dirname(args.inputfilename[0])
plot_folder = os.path.join(dirname, 'plots', filename)
if(not os.path.exists(plot_folder)):
    os.makedirs(plot_folder)

fin = h5py.File(args.inputfilename[0], 'r')
#weights = np.array(fin['weights'])[np.array(fin['weights']) > 1e-10]
weights = np.array(fin['weights'])
triggered = np.array(fin['triggered'])
xx = np.array(fin['xx'])
yy = np.array(fin['yy'])
zz = np.array(fin['zz'])
receive_vectors = np.array(fin['receive_vectors'])
theta = np.array(fin['zeniths'])
phi = np.array(fin['azimuths'])
polarization = np.array(fin['polarization']).flatten()
launch_vectors = np.array(fin['launch_vectors'])
n_events = fin.attrs['n_events']

for i in range(len(args.inputfilename) - 1):
    fin = h5py.File(args.inputfilename[i + 1], 'r')
    weights = np.append(weights, np.array(fin['weights']))
    triggered = np.append(triggered, np.array(fin['triggered']))
    xx = np.append(xx, np.array(fin['xx']))
    yy = np.append(yy, np.array(fin['yy']))
    zz = np.append(zz, np.array(fin['zz']))
    receive_vectors = np.append(receive_vectors, np.array(fin['receive_vectors']), axis=0)
    theta = np.append(theta, np.array(fin['zeniths']))
    phi = np.append(phi, np.array(fin['azimuths']))
    polarization = np.append(polarization, np.array(fin['polarization']).flatten())
    launch_vectors = np.append(launch_vectors, np.array(fin['launch_vectors']), axis=0)
    n_events += fin.attrs['n_events']

# calculate effective volume
density_ice = 0.9167 * units.g / units.cm ** 3
density_water = 997 * units.kg / units.m ** 3
n_triggered = np.sum(weights[triggered])
print('total number of triggered events = ' + str(len(weights)))
print('fraction of triggered events = {:.0f}/{:.0f} = {:.3f}'.format(n_triggered, n_events, n_triggered / n_events))
V = None
if('xmax' in fin.attrs):
    dX = fin.attrs['xmax'] - fin.attrs['xmin']
    dY = fin.attrs['ymax'] - fin.attrs['ymin']
    dZ = fin.attrs['zmax'] - fin.attrs['zmin']
    V = dX * dY * dZ
elif('rmin' in fin.attrs):
    rmin = fin.attrs['rmin']
    rmax = fin.attrs['rmax']
    dZ = fin.attrs['zmax'] - fin.attrs['zmin']
    V = np.pi * (rmax**2 - rmin**2) * dZ
Veff = V * density_ice / density_water * 4 * np.pi * np.sum(weights[triggered]) / n_events
print("Veff = {:.6g} km^3 sr".format(Veff / units.km ** 3))

# plot vertex distribution rz
plt.subplot(2, 1, 1)
rr = (xx ** 2 + yy ** 2) ** 0.5
#h = ax.hist2d(rr / units.m, zz / units.m, bins=[np.arange(0, 4000, 100), np.arange(-3000, 0, 100)],
#              cmap=plt.get_cmap('Blues'), weights=weights)
plt.hist2d(rr / units.m, zz / units.m, bins=[np.arange(0, 9001, 100), np.arange(-3501, 0, 100)], cmap=plt.get_cmap('Blues'), weights=weights/n_events)
cb = plt.colorbar()
cb.set_label("weighted number of events")
plt.xlabel("r [m]")
plt.ylabel("z [m]")
plt.grid(True)
plt.subplot(2, 1, 2)
plt.hist2d(rr / units.m, zz / units.m, bins=[np.arange(0, 9001, 100), np.arange(-3501, 0, 100)], cmap=plt.get_cmap('Blues'), weights = np.ones(len(rr))/n_events)
cb = plt.colorbar()
cb.set_label("number of events")
plt.xlabel("r [m]")
plt.ylabel("z [m]")
plt.grid(True)
plt.suptitle("vertex distribution")
plt.savefig(os.path.join(plot_folder, 'vertex_distribution.pdf'), bbox_inches="tight")
plt.clf()

# plot incoming direction
# for all events, antennas and ray tracing solutions
zeniths, azimuths = hp.cartesian_to_spherical_vectorized(receive_vectors[:, :, :, 0].flatten(),
                                                         receive_vectors[:, :, :, 1].flatten(),
                                                         receive_vectors[:, :, :, 2].flatten())
for i in range(len(azimuths)):
    azimuths[i] = hp.get_normalized_angle(azimuths[i])
weights_matrix = np.outer(weights, np.ones(np.prod(receive_vectors.shape[1:-1]))).flatten()
mask = ~np.isnan(azimuths)  # exclude antennas with not ray tracing solution (or with just one ray tracing solution)
plt.subplot(1, 2, 1)
plt.hist(zeniths[mask] / units.deg, bins = np.arange(0, 181, 10), weights=weights_matrix[mask])
plt.xlabel('zenith [deg]')
plt.subplot(1, 2, 2)
plt.hist(azimuths[mask] / units.deg, bins = np.arange(0, 361, 45), weights=weights_matrix[mask])
plt.xlabel('azimuth [deg]')
plt.suptitle('weighted incoming signal direction')
plt.savefig(os.path.join(plot_folder, 'incoming_signal.pdf'), bbox_inches="tight")
plt.clf()

# plot direction distribution
plt.subplot(2, 1, 1)
plt.hist2d(phi / units.deg, theta / units.deg, bins=[np.arange(0, 361, 5), np.arange(0, 181, 5)], cmap=plt.get_cmap('Blues'), weights=weights/n_events)
#plt.gca().invert_yaxis()
cb = plt.colorbar()
cb.set_label("weighted number of events")
plt.xlabel("azimuth [deg]")
plt.ylabel("zenith [deg]")
plt.grid(True)
plt.subplot(2, 1, 2)
plt.hist2d(phi / units.deg, theta / units.deg, bins=[np.arange(0, 361, 5), np.arange(0, 181, 5)], cmap=plt.get_cmap('Blues'), weights = np.ones(len(phi))/n_events)
#plt.gca().invert_yaxis()
cb = plt.colorbar()
cb.set_label("number of events")
plt.xlabel("azimuth [deg]")
plt.ylabel("zenith [deg]")
plt.grid(True)
plt.suptitle("direction distribution")
plt.savefig(os.path.join(plot_folder, 'direction_distribution.pdf'), bbox_inches="tight")
plt.clf()

# plot polarization
mask_pol = ~np.isnan(polarization)
polarization = np.abs(polarization)
polarization[polarization > 90 * units.deg] = 180 * units.deg - polarization[polarization > 90 * units.deg]
bins = np.linspace(0, 90, 45)
# for all events, antennas and ray tracing solutions
mask = zeniths > 90 * units.deg  # select rays coming from below
plt.hist(polarization[mask_pol] / units.deg, bins=bins, weights=weights_matrix[mask_pol], color = 'b')
plt.xlabel('weighted polarization [deg]')
plt.hist(polarization[mask] / units.deg, bins=bins, weights=weights_matrix[mask], color = 'r')
plt.figtext(1.0, 0.5, "red: rays coming from below; N: " + str(len(polarization[mask])) + "\nblue: all; N: " + str(len(polarization)))
plt.savefig(os.path.join(plot_folder, 'polarization.pdf'), bbox_inches="tight")
plt.clf()

#plot neutrino direction
zeniths = theta / units.deg
plt.subplot(2, 1, 1)
plt.hist(zeniths, weights=weights, bins=np.arange(0, 181, 5))
plt.xlabel('zenith angle [deg]')
plt.ylabel('weighted entries')
avgZen = np.average(zeniths, weights=weights)
varZen = np.average((zeniths-avgZen)**2, weights=weights)
plt.figtext(1.0, 0.6, "N: " + str(len(zeniths)) + "\nmean: " + str(np.average(zeniths, weights=weights)) + "\nstd: " + str(varZen**0.5))
plt.subplot(2, 1, 2)
plt.hist(zeniths, bins=np.arange(0, 181, 5))
plt.xlabel('zenith angle [deg]')
plt.ylabel('unweighted entries')
plt.figtext(1.0, 0.2, "N: " + str(len(zeniths)) + "\nmean: " + str(np.average(zeniths)) + "\nstd: " + str(np.std(zeniths)))
plt.suptitle("neutrino direction")
plt.savefig(os.path.join(plot_folder, 'neutrino_direction.pdf'), bbox_inches="tight")
plt.clf()

#plot difference between cherenkov angle and viewing angle
# i.e., opposite to the direction of propagation. We need the propagation direction here, so we multiply the shower axis with '-1'
shower_axis = -1.0 * hp.spherical_to_cartesian(theta, phi)
viewing_angles_d = np.array([hp.get_angle(x, y) for x, y in zip(shower_axis, launch_vectors[:, 0, 0])])
viewing_angles_r = np.array([hp.get_angle(x, y) for x, y in zip(shower_axis, launch_vectors[:, 0, 1])])
# calculate correct chereknov angle for ice density at vertex position
ice = medium.southpole_simple()
n_indexs = np.array([ice.get_index_of_refraction(x) for x in np.array([xx, yy, zz]).T])
rho = np.arccos(1. / n_indexs)
weightsExt = weights
for chan in range(1, len(launch_vectors[0])):
    viewing_angles_d = np.append(viewing_angles_d, np.array([hp.get_angle(x, y) for x, y in zip(shower_axis, launch_vectors[:, chan, 0])]))
    viewing_angles_r = np.append(viewing_angles_r, np.array([hp.get_angle(x, y) for x, y in zip(shower_axis, launch_vectors[:, chan, 1])]))
    rho = np.append(rho, np.arccos(1. / n_indexs))
    weightsExt = np.append(weightsExt, weights)
dCherenkov_d = (viewing_angles_d - rho) / units.deg
dCherenkov_r = (viewing_angles_r - rho) / units.deg
mask_d = ~np.isnan(viewing_angles_d)
mask_r = ~np.isnan(viewing_angles_r)
plt.subplot(2, 1, 1)
plt.hist([dCherenkov_d[mask_d], dCherenkov_r[mask_r]], weights = [weightsExt[mask_d],  weightsExt[mask_r]], bins = np.arange(-30, 30, 1))
plt.xlabel('weighted viewing - cherenkov angle [deg]')
avgChe_d = np.average(dCherenkov_d[mask_d], weights = weightsExt[mask_d])
varChe_d = np.average((dCherenkov_d[mask_d] - avgChe_d)**2, weights = weightsExt[mask_d])
avgChe_r = np.average(dCherenkov_r[mask_r], weights = weightsExt[mask_r])
varChe_r = np.average((dCherenkov_r[mask_r] - avgChe_r)**2, weights = weightsExt[mask_r])
plt.figtext(1.0, 0.6, "N: " + str(len(dCherenkov_d[mask_d])) + "; Blue: direct ray\nmean: " + str(np.average(dCherenkov_d[mask_d], weights=weightsExt[mask_d])) + "\nstd: " + str(varChe_d**0.5) + "\nN: " + str(len(dCherenkov_r[mask_r])) + "; Red: refracted ray\nmean: " + str(np.average(dCherenkov_r[mask_r], weights=weightsExt[mask_r])) + "\nstd: " + str(varChe_r**0.5))
plt.subplot(2, 1, 2)
plt.hist([dCherenkov_d[mask_d], dCherenkov_r[mask_r]], bins = np.arange(-30, 30, 1))
plt.xlabel('unweighted viewing - cherenkov angle [deg]')
plt.figtext(1.0, 0.2, "N: " + str(len(dCherenkov_d[mask_d])) + "; Blue: direct ray\nmean: " + str(np.average(dCherenkov_d[mask_d])) + "\nstd: " + str(np.std(dCherenkov_d[mask_d])) + "\nN: " + str(len(dCherenkov_r[mask_r])) + "; Red: refracted ray\nmean: " + str(np.average(dCherenkov_r[mask_r])) + "\nstd: " + str(np.std(dCherenkov_r[mask_r])))
plt.savefig(os.path.join(plot_folder, 'dCherenkov.pdf'), bbox_inches="tight")
plt.clf()

#plot viewing_refracted vs viewing_direct
plt.scatter(viewing_angles_d / units.deg, viewing_angles_r / units.deg, 10)
plt.xlabel("direct viewing angle")
plt.ylabel("refracted viewing angle")
plt.axis([20, 110, 20, 110])
plt.grid(True)
plt.savefig(os.path.join(plot_folder, 'viewDvsviewR.pdf'), bbox_inches="tight")

# plot C0 parameter
# C0s = np.array(fin['ray_tracing_C0'])
# php.get_histogram(C0s.flatten())
# fig.suptitle('incoming signal direction')
#plt.show()



import numpy as np
cimport numpy as np
from operator import itemgetter
import time

# Numpy must be initialized. When using numpy from C or Cython you must
# _always_ do that, or you will have segfaults
np.import_array()

cdef extern from "numpy/arrayobject.h":
    void PyArray_ENABLEFLAGS(np.ndarray arr, int flags)

cdef extern from "analytic_raytracing.cpp":
    void find_solutions2(double * &, double * &, int * &, int & , double, double, double, double, double, double, double)
    double get_attenuation_along_path2(double, double, double, double, double, double, double, double, double, int)
    

cpdef find_solutions(x1, x2, n_ice, delta_n, z_0):
    cdef:
        double * C0s
        double * C1s
        int * types
        int size
        np.npy_intp shape[1]

#     t = time.time()
    find_solutions2(C0s, C1s, types, size, x1[0], x1[1], x2[0], x2[1], n_ice, delta_n, z_0)
#     print((time.time() - t) * 1000)

    # 1. Make sure that you have called np.import_array()
    # http://gael-varoquaux.info/programming/
    # cython-example-of-exposing-c-computed-arrays-in-python-without-data-copies.html
    # 2. OWNDATA flag is important. It tells the NumPy to free data when the python object is deleted.
    # https://stackoverflow.com/questions/23872946/force-numpy-ndarray-to-take-ownership-of-its-memory-in-cython/
    # You can verify that the memory gets freed when Python object is deleted by using tools such as pmap.
    shape[0] = < np.npy_intp > size
    cdef np.ndarray[double, ndim = 1] C0ss = np.PyArray_SimpleNewFromData(1, shape, np.NPY_DOUBLE, C0s)
    PyArray_ENABLEFLAGS(C0ss, np.NPY_OWNDATA)
    cdef np.ndarray[double, ndim = 1] C1ss = np.PyArray_SimpleNewFromData(1, shape, np.NPY_DOUBLE, C1s)
    PyArray_ENABLEFLAGS(C1ss, np.NPY_OWNDATA)
    cdef np.ndarray[int, ndim = 1] typess = np.PyArray_SimpleNewFromData(1, shape, np.NPY_INT, types)
    PyArray_ENABLEFLAGS(typess, np.NPY_OWNDATA)

    solutions = []
    for i in range(len(C0ss)):
        solutions.append({'type': typess[i],
                          'C0': C0ss[i],
                          'C1': C1ss[i]})

    s = sorted(solutions, key=itemgetter('type'))
#     print((time.time() - t) * 1000)
    return s


cpdef get_attenuation_along_path(x1, x2, C0, frequency, n_ice, delta_n, z_0, model):

#     t = time.time()
    return get_attenuation_along_path2(x1[0], x1[1], x2[0], x2[1], C0, frequency, n_ice, delta_n, z_0, model)
#     print((time.time() - t) * 1000)



import numpy as np
from matplotlib import pyplot as plt
from NuRadioMC.utilities import fluxes
from NuRadioMC.examples.Sensitivities import E2_fluxes3 as limit
from NuRadioMC.utilities import units
import json
legendfontsize = 11

# PyREx 1.5 sigma 60 m (trigger level, km3sr)
# En	Veff		Error
pyrex_60 = np.array((
#[15.5,	8.031E-3,	3.583E-3],
[16.0,	6.989E-2,	9.930E-3],
[16.5,	3.711E-1,	2.965E-2],
[17.0,	1.290E+0,	5.528E-2],
[17.5,	3.250E+0,	1.396E-1],
[18.0,	7.013E+0,	2.509E-1],
[18.5,	1.249E+1,	4.185E-1],
[19.0,	2.290E+1,	5.823E-1],
[19.5,	4.273E+1,	9.290E-1],
#[20.0,	6.479E+1,	1.144E+0]
))

pyrex_60[:,0] = 10**pyrex_60[:,0] *units.eV
pyrex_60[:,1] *= units.km**3
pyrex_60[:,2] *= units.km**3


# PyREx 1.5 sigma 100 m (trigger level, km3sr)
# En	Veff		Error
pyrex_100 = np.array((
#[15.5,	9.033E-3,	4.249E-3],
[16.0,	6.816E-2,	9.530E-3],
[16.5,	5.060E-1,	3.370E-2],
[17.0,	1.717E+0,	6.563E-2],
[17.5,	4.931E+0,	1.719E-1],
[18.0,	1.056E+1,	3.163E-1],
[18.5,	1.971E+1,	5.258E-1],
[19.0,	3.530E+1,	7.036E-1],
[19.5,	6.863E+1,	1.177E+0],
#[20.0,	1.083E+2,	1.519E+0]
))

pyrex_100[:,0] = 10**pyrex_100[:,0] *units.eV
pyrex_100[:,1] *= units.km**3
pyrex_100[:,2] *= units.km**3

#

fig, ax = limit.get_E2_limit_figure(diffuse = True,
                    show_ice_cube_EHE_limit=True,
                    show_ice_cube_HESE_fit=False,
                    show_ice_cube_HESE_data=True,
                    show_ice_cube_mu=True,
                    show_anita_I_III_limit=True,
                    show_auger_limit=True,
                    show_neutrino_best_fit=True,
                    show_neutrino_best_case=True,
                    show_neutrino_worst_case=True,
                    show_ara=True,
                    show_grand_10k=True,
                    show_grand_200k=False,
                    show_radar=False)
labels = []
labels = limit.add_limit(ax, labels,
                         pyrex_60[:,0],pyrex_60[:,1]
                         , 61, 'RNO (trigger)', livetime=5*units.year, linestyle='-', color='r',linewidth=3)

labels = limit.add_limit(ax, labels,
                         pyrex_100[:,0],pyrex_100[:,1]
                         , 61, 'RNO 100m (trigger)', livetime=5*units.year, linestyle='--', color='r',linewidth=3)


plt.legend(handles=labels, loc=2,fontsize= legendfontsize)
ax.set_aspect('equal')
fig.tight_layout()

fig.savefig("Diffuse_proposal.pdf")

plt.show()



from __future__ import absolute_import, division, print_function
import numpy as np
from NuRadioMC.utilities import units
import argparse
import h5py

parser = argparse.ArgumentParser(description='Run NuRadioMC simulation')
parser.add_argument('hdf5input', type=str,
                    help='path to NuRadioMC hdf5 input event list')
parser.add_argument('ASCIIoutput', type=str,
                    help='path to ASCII output file')
args = parser.parse_args()

fin = h5py.File(args.hdf5input, 'r')

event_ids = fin['event_ids']
flavors = fin['flavors']
energies = fin['energies']
ccncs = fin['interaction_type']
xx = fin['xx']
yy = fin['yy']
zz = fin['zz']
zeniths = fin['zeniths']
azimuths = fin['azimuths']
inelasticity = fin['inelasticity']

n_events = len(event_ids)

with open(args.ASCIIoutput, 'w') as fout:
    fout.write(fin.attrs['header'])
    for i in range(n_events):
        fout.write("{:08d} {:>+5d}  {:.5e}  {:s}  {:>10.3f}  {:>10.3f}  {:>10.3f}  {:>10.3f}  {:>10.3f}  {:>10.3f}\n".format(event_ids[i], flavors[i], energies[i], ccncs[i], xx[i], yy[i], zz[i], zeniths[i], azimuths[i], inelasticity[i]))
    fout.close()



from __future__ import absolute_import, division, print_function
import numpy as np
from radiotools import helper as hp
from radiotools import plthelpers as php
from matplotlib import pyplot as plt
from NuRadioMC.utilities import units
import h5py
import argparse
import json
import time
import os

parser = argparse.ArgumentParser(description='Plot NuRadioMC event list input')
parser.add_argument('inputfilename', type=str,
                    help='path to NuRadioMC hdf5 simulation input')
# parser.add_argument('outputfilename', type=str,
#                     help='name of output file storing the electric field traces at detector positions')
args = parser.parse_args()

fin = h5py.File(args.inputfilename, 'r')
# plot vertex distribution
fig, ax = plt.subplots(1, 1)
xx = np.array(fin['xx'])
yy = np.array(fin['yy'])
rr = (xx ** 2 + yy ** 2) ** 0.5
zz = np.array(fin['zz'])
h = ax.hist2d(rr / units.m, zz / units.m, bins=[np.arange(0, 4000, 100), np.arange(-3000, 0, 100)],
              cmap=plt.get_cmap('Blues'))
cb = plt.colorbar(h[3], ax=ax)
cb.set_label("number of events")
ax.set_aspect('equal')
ax.set_xlabel("r [m]")
ax.set_ylabel("z [m]")
fig.tight_layout()
plt.title('vertex distribution')
plt.savefig("output/simInputVertex.pdf")

# plot incoming direction
zeniths = np.array(fin['zeniths'])
azimuths = np.array(fin['azimuths'])
fig, axs = php.get_histograms([zeniths / units.deg, azimuths / units.deg],
                              bins=[np.arange(0, 181, 10), np.arange(0, 361, 45)],
                              xlabels=['zenith [deg]', 'azimuth [deg]'],
                              stats=False)
fig.suptitle('neutrino direction')
plt.title('incoming direction')
plt.savefig("output/simInputIncoming.pdf")

# plot inelasticity
inelasticity = np.array(fin['inelasticity'])
fig, axs = php.get_histogram(inelasticity,
                             bins=np.logspace(np.log10(0.0001), np.log10(1.0), 50),
                             xlabel='inelasticity', figsize=(6, 6),
                             stats=True)
axs.semilogx(True)
plt.title('inelasticity')
plt.savefig("output/simInputInelasticity.pdf")



import matplotlib.pyplot as plt
import numpy as np
import time
from NuRadioMC.SignalProp import analyticraytraycing as ray
from NuRadioMC.utilities import units, medium
import logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger('raytracing')

x1 = [-2 * units.km, -1500. * units.m]  # pulser position
x2 = [0., -200. * units.m]  # ARA antanna

r = ray.ray_tracing_2D(medium.southpole_simple())
solution = r.find_solutions(x1, x2)
fig, ax = plt.subplots(1, 1)
yyy, zzz = r.get_path(x1, x2, solution[0]['C0'])
ax.plot(yyy / units.m, zzz / units.m)
yyy, zzz = r.get_path(x1, x2, solution[1]['C0'])
ax.plot(yyy / units.m, zzz / units.m)
ax.set_ylim(-1600, 0)
ax.set_xlabel('x [m]')
ax.set_ylabel('z [m]')
fig.tight_layout()
fig.savefig("plots/IceCubePulserToARA1.png")

x1 = [-4 * units.km, -1500. * units.m]  # pulser position
x2 = [0., -200. * units.m]  # ARA antanna

r = ray.ray_tracing_2D(medium.southpole_simple())
solution = r.find_solutions(x1, x2)
fig, ax = plt.subplots(1, 1)
yyy, zzz = r.get_path(x1, x2, solution[0]['C0'])
ax.plot(yyy / units.m, zzz / units.m)
yyy, zzz = r.get_path(x1, x2, solution[1]['C0'])
ax.plot(yyy / units.m, zzz / units.m)
ax.set_ylim(-1600, 0)
ax.set_xlabel('x [m]')
ax.set_ylabel('z [m]')
fig.tight_layout()
fig.savefig("plots/IceCubePulserToARA2.png")
plt.show()




import glob
import os
import sys
import numpy as np
from NuRadioMC.utilities.hdf5_manipulator import merge

"""
merges multiple hdf5 output files into one single files.
The merger module automatically keeps track of the total number
of simulated events (which are needed to correctly calculate the effective volume).

The script expects that the folder structure is
../output/energy/*.hdf5.part????
"""
if(len(sys.argv) != 2):
    print("usage: python merge_hdf5.py /path/to/simulation/output/folder")
else:
    filenames = glob.glob("{}/*/*.hdf5.part????".format(sys.argv[1]))
    filenames2 = []
    for i, filename in enumerate(filenames):
        filename, ext = os.path.splitext(filename)
        if(ext != '.hdf5'):
            if(filename not in filenames2):
                d = os.path.split(filename)
                a, b = os.path.split(d[0])
                filenames2.append(filename)

    for filename in filenames2:
        if(os.path.splitext(filename)[1] == '.hdf5'):
            d = os.path.split(filename)
            a, b = os.path.split(d[0])
            output_filename = os.path.join(a, d[1])  #remove subfolder from filename
            if(os.path.exists(output_filename)):
                print('file {} already exists, skipping'.format(output_filename))
            else:
#                 try:
                    input_files = np.array(sorted(glob.glob(filename + '.part????')))
                    mask = np.array([os.path.getsize(x) > 1000 for x in input_files], dtype=np.bool)
                    if(np.sum(~mask)):
                        print("{:d} files were deselected because their filesize was to small".format(np.sum(~mask)))


                    merge.merge_data_filenames(input_files[mask], output_filename)
#                 except:
#                     print("failed to merge {}".format(filename))



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function
import numpy as np
from NuRadioMC.utilities import units, fft
from NuRadioMC.SignalGen import parametrizations as par
import logging
logger = logging.getLogger("SignalGen.askaryan")

gARZ = None

def set_log_level(level):
    logger.setLevel(level)
    par.set_log_level(level)

def get_time_trace(energy, theta, N, dt, shower_type, n_index, R, model, interp_factor=None, interp_factor2=None,
                   same_shower=False, **kwargs):
    """
    returns the Askaryan pulse in the time domain of the eTheta component

    We implement only the time-domain solution and obtain the frequency spectrum
    via FFT (with the standard normalization of NuRadioMC). This approach assures
    that the units are interpreted correctly. In the time domain, the amplitudes
    are well defined and not details about fourier transform normalizations needs
    to be known by the user.

    Parameters
    ----------
    energy : float
        energy of the shower
    theta: float
        viewangle: angle between shower axis (neutrino direction) and the line
        of sight between interaction and detector
    N : int
        number of samples in the time domain
    dt: float
        time bin width, i.e. the inverse of the sampling rate
    shower_type: string (default "HAD")
        type of shower, either "HAD" (hadronic), "EM" (electromagnetic) or "TAU" (tau lepton induced)
        note that TAU showers are currently only implemented in the ARZ2019 model
    n_index: float
        index of refraction at interaction vertex
    R: float
        distance from vertex to observer
    model: string
        specifies the signal model
        * ZHS1992: the original ZHS parametrization from E. Zas, F. Halzen, and T. Stanev, Phys. Rev. D 45, 362 (1992), doi:10.1103/PhysRevD.45.362, this parametrization does not contain any phase information
        * Alvarez2000: parameterization based on ZHS mainly based on J. Alvarez-Muniz, R. A. V ́azquez, and E. Zas, Calculation methods for radio pulses from high energyshowers, Physical Review D62 (2000) https://doi.org/10.1103/PhysRevD.84.103003
        * Alvarez2009: parameterization based on ZHS from J. Alvarez-Muniz, W. R. Carvalho, M. Tueros, and E. Zas, Coherent cherenkov radio pulses fromhadronic showers up to EeV energies, Astroparticle Physics 35 (2012), no. 6 287 – 299 and J. Alvarez-Muniz, C. James, R. Protheroe, and E. Zas, Thinned simulations of extremely energeticshowers in dense media for radio applications, Astroparticle Physics 32 (2009), no. 2 100 – 111
        * HCRB2017: analytic model from J. Hanson, A. Connolly Astroparticle Physics 91 (2017) 75-89
        * ARZ2019 semi MC time domain model from Alvarez-Muñiz, J., Romero-Wolf, A., & Zas, E. (2011). Practical and accurate calculations of Askaryan radiation. Physical Review D - Particles, Fields, Gravitation and Cosmology, 84(10). https://doi.org/10.1103/PhysRevD.84.103003

    interp_factor: float or None
        controls the interpolation of the charge-excess profiles in the ARZ model
    interp_Factor2: float or None
        controls the second interpolation of the charge-excess profiles in the ARZ model
    same_shower: bool (default False)
        controls the random behviour of picking a shower from the library in the ARZ model, see description there for
        more details

    Returns
    -------
    spectrum: array
        the complex amplitudes for the given frequencies

    """
    if model in par.get_parametrizations():
        return par.get_time_trace(energy, theta, N, dt, shower_type, n_index, R, model)
    elif(model == 'HCRB2017'):
        from NuRadioMC.SignalGen import HCRB2017
        is_em_shower = None
        if(shower_type == "HAD"):
            is_em_shower = False
        elif(shower_type == "EM"):
            is_em_shower = True
        else:
            raise NotImplementedError("shower type {} not implemented in {} Askaryan module".format(shower_type, model))
        LPM = True
        a = None
        if('LPM' in kwargs):
            LPM = kwargs['LPM']
        if('a' in kwargs):
            a = kwargs['a']
        return HCRB2017.get_time_trace(energy, theta, N, dt, is_em_shower, n_index, R, LPM, a)[1]
    elif(model == 'ARZ2019'):
        from NuRadioMC.SignalGen.ARZ import ARZ
        global gARZ
        if(gARZ is None):
            gARZ = ARZ.ARZ()
        if(interp_factor is not None):
            gARZ.set_interpolation_factor(interp_factor)

        if(interp_factor2 is not None):
            gARZ.set_interpolation_factor2(interp_factor2)
        return gARZ.get_time_trace(energy, theta, N, dt, shower_type, n_index, R, same_shower=same_shower, **kwargs)[1]

    elif(model == 'spherical'):
        amplitude = 1. * energy / R
        trace = np.zeros(N)
        trace[N//2] = amplitude
        return trace
    else:
        raise NotImplementedError("model {} unknown".format(model))


def get_frequency_spectrum(energy, theta, N, dt, is_em_shower, n_index, R, model, **kwargs):
    """
    returns the complex amplitudes of the frequency spectrum of the neutrino radio signal

    Parameters
    ----------
    energy : float
        energy of the shower
    theta: float
        viewangle: angle between shower axis (neutrino direction) and the line
        of sight between interaction and detector
    N : int
        number of samples in the time domain
    dt: float
        time bin width, i.e. the inverse of the sampling rate
    is_em_shower: bool
        true if EM shower, false otherwise
    n_index: float
        index of refraction at interaction vertex
    R: float
        distance from vertex to observer
    model: string
        specifies the signal model
        * ZHS1992: the original ZHS parametrization from E. Zas, F. Halzen, and T. Stanev, Phys. Rev. D 45, 362 (1992), doi:10.1103/PhysRevD.45.362, this parametrization does not contain any phase information
        * Alvarez2000: parameterization based on ZHS mainly based on J. Alvarez-Muniz, R. A. V ́azquez, and E. Zas, Calculation methods for radio pulses from high energyshowers, Physical Review D62 (2000) https://doi.org/10.1103/PhysRevD.84.103003
        * Alvarez2009: parameterization based on ZHS from J. Alvarez-Muniz, W. R. Carvalho, M. Tueros, and E. Zas, Coherent cherenkov radio pulses fromhadronic showers up to EeV energies, Astroparticle Physics 35 (2012), no. 6 287 – 299 and J. Alvarez-Muniz, C. James, R. Protheroe, and E. Zas, Thinned simulations of extremely energeticshowers in dense media for radio applications, Astroparticle Physics 32 (2009), no. 2 100 – 111
        * HCRB2017: analytic model from J. Hanson, A. Connolly Astroparticle Physics 91 (2017) 75-89
        * ARZ2019 semi MC time domain model from Alvarez-Muñiz, J., Romero-Wolf, A., & Zas, E. (2011). Practical and accurate calculations of Askaryan radiation. Physical Review D - Particles, Fields, Gravitation and Cosmology, 84(10). https://doi.org/10.1103/PhysRevD.84.103003
    Returns
    -------
    spectrum: array
        the complex amplitudes for the given frequencies

    """
    return fft.time2freq(get_time_trace(energy, theta, N, dt, is_em_shower, n_index, R, model, **kwargs))



from __future__ import absolute_import, division, print_function
import numpy as np
from radiotools import helper as hp
from radiotools import plthelpers as php
from matplotlib import pyplot as plt
from NuRadioMC.utilities import units
from NuRadioMC.utilities import medium
from NuRadioMC.utilities import plotting
from six import iteritems
import h5py
import argparse
import json
import time
import os
from matplotlib.ticker import MultipleLocator, FormatStrFormatter

parser = argparse.ArgumentParser(description='Plot NuRadioMC event list output.')
parser.add_argument('inputfilename', type=str,
                    help='path to NuRadioMC hdf5 simulation output')
parser.add_argument('--trigger_name', type=str, default=None, nargs='+',
                    help='the name of the trigger that should be used for the plots')
parser.add_argument('--Veff', type=str,
                    help='specify json file where effective volume is saved as a function of energy')
args = parser.parse_args()

filename = os.path.splitext(os.path.basename(args.inputfilename))[0]
dirname = os.path.dirname(args.inputfilename)
plot_folder = os.path.join(dirname, 'plots', filename)
if(not os.path.exists(plot_folder)):
    os.makedirs(plot_folder)

fin = h5py.File(args.inputfilename, 'r')
print('the following triggeres where simulated: {}'.format(fin.attrs['trigger_names']))
if(args.trigger_name is None):
    triggered = np.array(fin['triggered'])
    print("you selected any trigger")
    trigger_name = 'all'
else:
    if(len(args.trigger_name) > 1):
        print("trigger {} selected which is a combination of {}".format(args.trigger_name[0], args.trigger_name[1:]))
        trigger_name = args.trigger_name[0]
        plot_folder = os.path.join(dirname, 'plots', filename, args.trigger_name[0])
        if(not os.path.exists(plot_folder)):
            os.makedirs(plot_folder)
        triggered = np.zeros(len(fin['multiple_triggers'][:, 0]), dtype=np.bool)
        for trigger in args.trigger_name[1:]:
            iTrigger = np.squeeze(np.argwhere(fin.attrs['trigger_names'] == trigger))
            triggered = triggered | np.array(fin['multiple_triggers'][:, iTrigger], dtype=np.bool)
    else:
        trigger_name = args.trigger_name[0]
        iTrigger = np.argwhere(fin.attrs['trigger_names'] == trigger_name)
        triggered = np.array(fin['multiple_triggers'][:, iTrigger], dtype=np.bool)
        print("\tyou selected '{}'".format(trigger_name))
        plot_folder = os.path.join(dirname, 'plots', filename, trigger_name)
        if(not os.path.exists(plot_folder)):
            os.makedirs(plot_folder)

weights = np.array(fin['weights'])[triggered]
n_events = fin.attrs['n_events']

###########################
# calculate effective volume
###########################
density_ice = 0.9167 * units.g / units.cm ** 3
density_water = 997 * units.kg / units.m ** 3

n_triggered = np.sum(weights)
print('fraction of triggered events = {:.0f}/{:.0f} = {:.3f}'.format(n_triggered, n_events, n_triggered / n_events))

V = None
if('xmax' in fin.attrs):
    dX = fin.attrs['xmax'] - fin.attrs['xmin']
    dY = fin.attrs['ymax'] - fin.attrs['ymin']
    dZ = fin.attrs['zmax'] - fin.attrs['zmin']
    V = dX * dY * dZ
elif('rmin' in fin.attrs):
    rmin = fin.attrs['rmin']
    rmax = fin.attrs['rmax']
    dZ = fin.attrs['zmax'] - fin.attrs['zmin']
    V = np.pi * (rmax**2 - rmin**2) * dZ
Veff = V * density_ice / density_water * 4 * np.pi * np.sum(weights) / n_events
print("Veff = {:.6g} km^3 sr".format(Veff / units.km ** 3))

 
###########################
# plot neutrino direction
###########################
fig, ax = php.get_histogram(np.array(fin['zeniths'])[triggered] / units.deg, weights=weights,
                            ylabel='weighted entries', xlabel='zenith angle [deg]',
                            bins=np.arange(0, 181, 5), figsize=(6, 6))
ax.set_xticks(np.arange(0, 181, 45))
ax.set_title(trigger_name)
fig.tight_layout()
fig.savefig(os.path.join(plot_folder, 'neutrino_direction.png'))

###########################
# calculate sky coverage of 90% quantile
###########################
from radiotools import stats
q2 =stats.quantile_1d(np.array(fin['zeniths'])[triggered], weights, 0.95)
q1 =stats.quantile_1d(np.array(fin['zeniths'])[triggered], weights, 0.05)
from scipy import integrate
def a(theta):
    return np.sin(theta)
b = integrate.quad(a, q1, q2)
print("90% quantile sky coverage {:.2f} sr".format(b[0] * 2 * np.pi))

###########################
# plot vertex distribution
###########################


xx = np.array(fin['xx'])[triggered]
yy = np.array(fin['yy'])[triggered]
zz = np.array(fin['zz'])[triggered]
fig, ax = plotting.plot_vertex_distribution(xx, yy, zz, weights=weights, rmax=rmax, zmin=fin.attrs['zmin'],
                                           trigger_name=trigger_name)
fig.savefig(os.path.join(plot_folder, 'vertex_distribution.png'), bbox='tight')


###########################
# loop over all stations and produce station specific plots
###########################

for key, station in iteritems(fin):
    if isinstance(station, h5py._hl.group.Group):
        ###########################
        # recalculate triggers per station
        ###########################
        
        if(args.trigger_name is None):
            triggered = np.array(station['triggered'])
            print("you selected any trigger")
            trigger_name = 'all'
        else:
            if(len(args.trigger_name) > 1):
                print("trigger {} selected which is a combination of {}".format(args.trigger_name[0], args.trigger_name[1:]))
                trigger_name = args.trigger_name[0]
                triggered = np.zeros(len(station['multiple_triggers'][:, 0]), dtype=np.bool)
                for trigger in args.trigger_name[1:]:
                    iTrigger = np.squeeze(np.argwhere(fin.attrs['trigger_names'] == trigger))
                    triggered = triggered | np.array(station['multiple_triggers'][:, iTrigger], dtype=np.bool)
            else:
                trigger_name = args.trigger_name[0]
                iTrigger = np.argwhere(fin.attrs['trigger_names'] == trigger_name)
                triggered = np.array(station['multiple_triggers'][:, iTrigger], dtype=np.bool)
                print("\tyou selected '{}'".format(trigger_name))
            
        
        ###########################
        # plot incoming direction
        ###########################
        receive_vectors = np.array(station['receive_vectors'])[triggered]
        # for all events, antennas and ray tracing solutions
        zeniths, azimuths = hp.cartesian_to_spherical(receive_vectors[:, :, :, 0].flatten(),
                                                      receive_vectors[:, :, :, 1].flatten(),
                                                      receive_vectors[:, :, :, 2].flatten())
        for i in range(len(azimuths)):
            azimuths[i] = hp.get_normalized_angle(azimuths[i])
        weights_matrix = np.outer(weights, np.ones(np.prod(receive_vectors.shape[1:-1]))).flatten()
        mask = ~np.isnan(azimuths)  # exclude antennas with not ray tracing solution (or with just one ray tracing solution)
        fig, axs = php.get_histograms([zeniths[mask] / units.deg, azimuths[mask] / units.deg],
                                      bins=[np.arange(0, 181, 5), np.arange(0, 361, 45)],
                                      xlabels=['zenith [deg]', 'azimuth [deg]'],
                                      weights=weights_matrix[mask], stats=False)
        # axs[0].xaxis.set_ticks(np.arange(0, 181, 45))
        majorLocator = MultipleLocator(45)
        majorFormatter = FormatStrFormatter('%d')
        minorLocator = MultipleLocator(5)
        axs[0].xaxis.set_major_locator(majorLocator)
        axs[0].xaxis.set_major_formatter(majorFormatter)
        axs[0].xaxis.set_minor_locator(minorLocator)
        
        fig.suptitle('incoming signal direction')
        fig.savefig(os.path.join(plot_folder, '{}_incoming_signal.png'.format(key)))
        
        ###########################
        # plot polarization
        ###########################
        p = np.array(station['polarization'])[triggered]
        p_H = (p[:,:,:,0]**2 + p[:,:,:,1]**2)**0.5
        p_V = np.abs(p[:,:,:,2])
        weights_matrix = np.outer(weights, np.ones(np.prod(p_V.shape[1:]))).flatten()
        p_ratio = (p_V/p_H).flatten()
        bins = np.linspace(0, 1, 50)
        
#         for all events, antennas and ray tracing solutions
        mask = zeniths > 90 * units.deg  # select rays coming from below
        fig, ax = php.get_histogram(p_ratio,
                                    bins=bins,
                                    xlabel='vertical/horizonal polarization ratio',
                                    weights=weights_matrix, stats=False,
                                    kwargs={'facecolor':'0.7', 'alpha':1, 'edgecolor':"k", 'label': 'all'},
                                    figsize=(6, 6))
        maxy = ax.get_ylim()
        php.get_histogram(p_ratio[mask],
                          bins=bins,
                          weights=weights_matrix[mask], stats=False,
                          xlabel='vertical/horizonal polarization ratio',
                          ax=ax, kwargs={'facecolor': 'C0', 'alpha': 1, 'edgecolor': "k", 'label': 'direct rays'})
        # ax.set_xticks(bins)
        ax.legend()
        ax.set_ylim(maxy)
        fig.tight_layout()
        fig.savefig(os.path.join(plot_folder, '{}_polarization.png'.format(key)))
       
        mask = zeniths > 90 * units.deg  # select rays coming from below
        fig, ax = php.get_histogram(p_ratio,
                                    bins=bins,
                                    stats=False,
                                    xlabel='vertical/horizonal polarization ratio',
                                    kwargs={'facecolor':'0.7', 'alpha':1, 'edgecolor':"k", 'label': 'all'},
                                    figsize=(6, 6))
        maxy = ax.get_ylim()
        php.get_histogram(p_ratio[mask],
                          bins=bins,
                          xlabel='vertical/horizonal polarization ratio',
                          stats=False,
                          ax=ax, kwargs={'facecolor': 'C0', 'alpha': 1, 'edgecolor': "k", 'label': 'direct rays'})
        # ax.set_xticks(bins)
        ax.legend()
        ax.set_ylim(maxy)
        fig.tight_layout()
        fig.savefig(os.path.join(plot_folder, '{}_polarization_unweighted.png'.format(key)))

        ###########################
        # plot viewing angle
        ###########################
        shower_axis = -1 * hp.spherical_to_cartesian(np.array(fin['zeniths'])[triggered], np.array(fin['azimuths'])[triggered])
        launch_vectors = np.array(station['launch_vectors'])[triggered]
        viewing_angles = np.array([hp.get_angle(x, y) for x, y in zip(shower_axis, launch_vectors[:, 0, 0])])
        
        # calculate correct chereknov angle for ice density at vertex position
        ice = medium.southpole_simple()
        n_indexs = np.array([ice.get_index_of_refraction(x) for x in np.array([np.array(fin['xx'])[triggered], np.array(fin['yy'])[triggered], np.array(fin['zz'])[triggered]]).T])
        rho = np.arccos(1. / n_indexs)
        
        mask = ~np.isnan(viewing_angles)
        fig, ax = php.get_histogram((viewing_angles[mask] - rho[mask]) / units.deg, weights=weights[mask],
                                    bins=np.arange(-30, 30, 1), xlabel='viewing - cherenkov angle [deg]', figsize=(6, 6))
        fig.savefig(os.path.join(plot_folder, '{}_dCherenkov.png'.format(key)))
        
        ###########################
        # plot flavor ratios
        ###########################
        flavor_labels = ['e cc', r'$\bar{e}$ cc', 'e nc', r'$\bar{e}$ nc',
                   '$\mu$ cc', r'$\bar{\mu}$ cc', '$\mu$ nc', r'$\bar{\mu}$ nc',
                   r'$\tau$ cc', r'$\bar{\tau}$ cc', r'$\tau$ nc', r'$\bar{\tau}$ nc']
        yy = np.zeros(len(flavor_labels))
        yy[0] = np.sum(weights[(fin['flavors'][triggered] == 12) & (fin['interaction_type'][triggered] == 'cc')])
        yy[1] = np.sum(weights[(fin['flavors'][triggered] == -12) & (fin['interaction_type'][triggered] == 'cc')])
        yy[2] = np.sum(weights[(fin['flavors'][triggered] == 12) & (fin['interaction_type'][triggered] == 'nc')])
        yy[3] = np.sum(weights[(fin['flavors'][triggered] == -12) & (fin['interaction_type'][triggered] == 'nc')])
        
        yy[4] = np.sum(weights[(fin['flavors'][triggered] == 14) & (fin['interaction_type'][triggered] == 'cc')])
        yy[5] = np.sum(weights[(fin['flavors'][triggered] == -14) & (fin['interaction_type'][triggered] == 'cc')])
        yy[6] = np.sum(weights[(fin['flavors'][triggered] == 14) & (fin['interaction_type'][triggered] == 'nc')])
        yy[7] = np.sum(weights[(fin['flavors'][triggered] == -14) & (fin['interaction_type'][triggered] == 'nc')])
        
        yy[8] = np.sum(weights[(fin['flavors'][triggered] == 16) & (fin['interaction_type'][triggered] == 'cc')])
        yy[9] = np.sum(weights[(fin['flavors'][triggered] == -16) & (fin['interaction_type'][triggered] == 'cc')])
        yy[10] = np.sum(weights[(fin['flavors'][triggered] == 16) & (fin['interaction_type'][triggered] == 'nc')])
        yy[11] = np.sum(weights[(fin['flavors'][triggered] == -16) & (fin['interaction_type'][triggered] == 'nc')])
        
        fig, ax = plt.subplots(1, 1, figsize=(8, 6))
        ax.bar(range(len(flavor_labels)), yy)
        ax.set_xticks(range(len(flavor_labels)))
        ax.set_xticklabels(flavor_labels, fontsize='large', rotation=45)
        ax.set_title("trigger: {}".format(trigger_name))
        ax.set_ylabel('weighted number of triggers', fontsize='large')
        fig.tight_layout()
        fig.savefig(os.path.join(plot_folder, '{}_flavor.png'.format(key)))
        plt.show()




import numpy as np
from NuRadioMC.utilities import units
from scipy.interpolate import interp1d


def param(energy, inttype='cc'):
    """
    Parameterization and constants as used in
    get_nu_cross_section()
    See documentation there for details

    """

    if inttype == 'cc':
        c = (-1.826, -17.31, -6.406, 1.431, -17.91) # nu, CC
    elif inttype == 'nc':
        c = (-1.826, -17.31, -6.448, 1.431, -18.61) # nu, NC
    elif inttype == 'cc_bar':
        c = (-1.033, -15.95, -7.247, 1.569, -17.72) # nu_bar, CC
    elif inttype == 'nc_bar':
        c = (-1.033, -15.95, -7.296, 1.569, -18.30) # nu_bar, NC
    else:
        logger.error("Type {0} of interaction not defined".format(inttype))
        raise NotImplementedError

    epsilon = np.log10(energy/units.GeV)
    l_eps = np.log(epsilon - c[0])
    crscn = c[1] + c[2] * l_eps + c[3] * l_eps**2 + c[4]/l_eps
    crscn = np.power(10,crscn) * units.cm**2
    return crscn


def csms(energy, inttype, flavors):
    """
    Neutrino cross sections according to
    Amanda Cooper-Sarkar, Philipp Mertsch, Subir Sarkar
    JHEP 08 (2011) 042
    """
    if type(inttype) == str:
        inttype = np.array([inttype]*energy.shape[0])

    if ( type(flavors) == int or type(flavors) == np.int64 ):
        flavors = np.array([flavors]*energy.shape[0])

    neutrino = np.array((
        [50,0.32,0.10],
        [100,0.65,0.20],
        [200,1.3,0.41],
        [500,3.2,1.0],
        [1000,6.2,2.0],
        [2000,12.,3.8],
        [5000,27.,8.6],
        [10000,47.,15.],
        [20000,77.,26.],
        [50000,140.,49.],
        [100000,210.,75.],
        [200000,310.,110.],
        [500000,490.,180.],
        [1e6,690.,260.],
        [2e6,950.,360.],
        [5e6,1400.,540.],
        [1e7,1900.,730.],
        [2e7,2600.,980.],
        [5e7,3700.,1400.],
        [1e8,4800.,1900.],
        [2e8,6200.,2400.],
        [5e8,8700.,3400.],
        [1e9,11000.,4400.],
        [2e9,14000.,5600.],
        [5e9,19000.,7600.],
        [1e10,24000.,9600.],
        [2e10,30000.,12000.],
        [5e10,39000.,16000.],
        [1e11,48000.,20000.],
        [2e11,59000.,24000.],
        [5e11,75000.,31000.]
        ))

    neutrino[:,0] *= units.GeV
    neutrino[:,1] *= units.picobarn #CC
    neutrino[:,2] *= units.picobarn #NC

    neutrino_cc = interp1d(neutrino[:,0], neutrino[:,1],bounds_error=True)
    neutrino_nc = interp1d(neutrino[:,0], neutrino[:,2],bounds_error=True)

    antineutrino = np.array((
        [50,0.15,0.05],
        [100,0.33,0.12],
        [200,0.69,0.24],
        [500,1.8,0.61],
        [1000,3.6,1.20],
        [2000,7.,2.4],
        [5000,17.,5.8],
        [10000,31.,11.],
        [20000,55.,19.],
        [50000,110.,39.],
        [100000,180.,64.],
        [200000,270.,99.],
        [500000,460.,170.],
        [1e6,660.,240.],
        [2e6,920.,350.],
        [5e6,1400.,530.],
        [1e7,1900.,730.],
        [2e7,2500.,980.],
        [5e7,3700.,1400.],
        [1e8,4800.,1900.],
        [2e8,6200.,2400.],
        [5e8,8700.,3400.],
        [1e9,11000.,4400.],
        [2e9,14000.,5600.],
        [5e9,19000.,7600.],
        [1e10,24000.,9600.],
        [2e10,30000.,12000.],
        [5e10,39000.,16000.],
        [1e11,48000.,20000.],
        [2e11,59000.,24000.],
        [5e11,75000.,31000.]
    ))

    antineutrino[:,0] *= units.GeV
    antineutrino[:,1] *= units.picobarn #CC
    antineutrino[:,2] *= units.picobarn #NC

    antineutrino_cc = interp1d(antineutrino[:,0], antineutrino[:,1],bounds_error=True)
    antineutrino_nc = interp1d(antineutrino[:,0], antineutrino[:,2],bounds_error=True)

    crscn = np.zeros_like(energy)

    particles_cc = np.where((flavors >= 0) & (inttype == 'cc'))
    particles_nc = np.where((flavors >= 0) & (inttype == 'nc'))
    antiparticles_cc = np.where((flavors < 0) & (inttype == 'cc'))
    antiparticles_nc = np.where((flavors < 0) & (inttype == 'nc'))
#
    crscn[particles_cc] = neutrino_cc(energy[particles_cc])
    crscn[particles_nc] = neutrino_nc(energy[particles_nc])
    crscn[antiparticles_cc] = antineutrino_cc(energy[antiparticles_cc])
    crscn[antiparticles_nc] = antineutrino_nc(energy[antiparticles_nc])

    return crscn


def get_nu_cross_section(energy, flavors, inttype='total', cross_section_type = 'ctw'):
    """
    return neutrino cross-section

    Parameters
        ----------
    energy: float/ array of floats
        neutrino energies/momenta in standard units

    flavors: float / array of floats
        neutrino flavor (integer) encoded as using PDG numbering scheme,
        particles have positive sign, anti-particles have negative sign, relevant are:
        12: electron neutrino
        14: muon neutrino
        16: tau neutrino

    inttype: str, array of str
        interaction type
        nc : neutral current
        cc : charged current
        total: total (for non-array type)

    cross_section_type: str
        defines model of cross-section
        ghandi : according to Ghandi et al. Phys.Rev.D58:093009,1998
                 only one cross-section for all interactions and flavors
        ctw    : A. Connolly, R. S. Thorne, and D. Waters, Phys. Rev.D 83, 113009 (2011).
                 cross-sections for all interaction types and flavors
        csms   : A. Cooper-Sarkar, P. Mertsch, S. Sarkar, JHEP 08 (2011) 042
    """


    if cross_section_type == 'ghandi':
        crscn = 7.84e-36 * units.cm**2 * np.power(energy/units.GeV,0.363)

    elif cross_section_type == 'ctw':
        crscn = np.zeros_like(energy)
        if type(inttype) == str:
            if inttype == 'total':

                if ( type(flavors) == int or type(flavors) == np.int64 ):
                    if flavors >= 0:
                        crscn = param(energy,'nc') + param(energy,'cc')
                    else:
                        crscn = param(energy,'nc_bar') + param(energy,'cc_bar')
                else:
                    antiparticles = np.where(flavors < 0)
                    particles = np.where(flavors >= 0)

                    crscn[particles] = param(energy[particles],'nc') + param(energy[particles],'cc')
                    crscn[antiparticles] = param(energy[antiparticles],'nc_bar') + param(energy[antiparticles],'cc_bar')

            if (inttype == 'cc') or (inttype =='nc'):
                if ( type(flavors) == int or type(flavors) == np.int64 ):
                    crscn = param(energy,inttype)
                else:
                    antiparticles = np.where(flavors < 0)
                    particles = np.where(flavors >= 0)
                    crscn[particles] = param(energy[particles],inttype)
                    crscn[antiparticles] = param(energy[antiparticles],inttype)
        else:

                if ( type(flavors) == int or type(flavors) == np.int64 ):

                    particles_cc = np.where(inttype == 'cc')
                    particles_nc = np.where(inttype == 'nc')
                    if flavors >= 0:
                        crscn[particles_cc] = param(energy[particles_cc],'cc')
                        crscn[particles_nc] = param(energy[particles_nc],'nc')
                    else:
                        crscn[particles_cc] = param(energy[particles_cc],'cc_bar')
                        crscn[particles_nc] = param(energy[particles_nc],'nc_bar')


                else:
                    particles_cc = np.where((flavors >= 0) & (inttype == 'cc'))
                    particles_nc = np.where((flavors >= 0) & (inttype == 'nc'))
                    antiparticles_cc = np.where((flavors < 0) & (inttype == 'cc'))
                    antiparticles_nc = np.where((flavors < 0) & (inttype == 'nc'))

                    crscn[particles_cc] = param(energy[particles_cc],'cc')
                    crscn[particles_nc] = param(energy[particles_nc],'nc')
                    crscn[antiparticles_cc] = param(energy[antiparticles_cc],'cc_bar')
                    crscn[antiparticles_nc] = param(energy[antiparticles_nc],'nc_bar')

    elif cross_section_type == 'csms':
        crscn = csms(energy, inttype, flavors)

    else:
        logger.error("Cross-section {} not defined".format(cross_section_type))
        raise NotImplementedError

    return crscn


if __name__=="__main__":  # this part of the code gets only executed it the script is directly called

    inttype = np.array(['cc']*10)

#     inttype = 'cc'

    flavors = np.array([14]*10)

#     flavors = 14

    energy = np.array([1e12,1e14,1e16,2e17,6e17,1e18,5e18,9e18,1e19,1e20])*units.eV

    cc = get_nu_cross_section(energy, flavors,inttype=inttype)/units.picobarn

    cc_new = get_nu_cross_section(energy, flavors, inttype=inttype, cross_section_type = 'csms')/units.picobarn

    import matplotlib.pyplot as plt

    plt.figure()
    plt.loglog(energy/units.GeV,cc,label='CTW')
    plt.loglog(energy/units.GeV,cc_new,label='CSMS')
    plt.xlabel("Energy [GeV]")
    plt.ylabel("CC [pb]")
    plt.legend()


    plt.show()



from __future__ import absolute_import, division, print_function
import numpy as np
from NuRadioMC.utilities import units
from NuRadioMC.utilities import cross_sections
import logging
logger = logging.getLogger("utilities.earth_attenuation")

R_earth = 6357390 * units.m
DensityCRUST = 2900 * units.kg / units.m ** 3
AMU = 1.66e-27 * units.kg
R_EARTH = 6.378140e6 * units.m
densities = np.array([14000.0, 3400.0, 2900.0]) * units.kg / units.m ** 3 # inner layer, middle layer, outer layer
radii = np.array([3.46e6 * units.m, R_EARTH - 4.0e4 * units.m, R_EARTH]) # average radii of boundaries between earth layers


def get_weight(theta_nu, pnu, flavors, mode='simple',cross_section_type = 'ghandi'):
    """
    calculates neutrino weight due to Earth absorption for different models

    Parameters
    ----------
    theta_nu: float or array of floats
        the zenith angle of the neutrino direction (where it came from, i.e., opposite to the direction of propagation)
    pnu: float or array of floats
        the momentum of the neutrino
    """
    if(mode == 'simple'):
        return get_simple_weight(theta_nu, pnu,cross_section_type = cross_section_type)
    elif (mode == "core_mantle_crust"):
        return get_core_mantle_crust_weight(theta_nu, pnu, flavors,cross_section_type = cross_section_type)
    elif (mode == "None"):
        return 1.
    else:
        logger.error('mode {} not supported'.format(mode))
        raise NotImplementedError


def get_simple_weight(theta_nu, pnu, cross_section_type = 'ghandi'):
    """
    calculates neutrino weight due to Earth absorption, i.e. probability of the
    neutrino to reach the detector

    simple parametrization using only momentum and zenith angle information
    of the neutrino, adapted from ShelfMC

    Parameters
    ----------
    theta_nu: float or array of floats
        the zenith angle of the neutrino direction (where it came from, i.e., opposite to the direction of propagation)
    pnu: float or array of floats
        the momentum of the neutrino
    """
    if(theta_nu <= 0.5 * np.pi):  # coming from above
        return np.ones_like(theta_nu)
    else:  # coming from below
        sigma = cross_sections.get_nu_cross_section(pnu, flavors = 0, cross_section_type = cross_section_type)
        d = - 2 * R_earth * np.cos(theta_nu)
        return np.exp(-d * sigma * DensityCRUST / AMU)

def get_core_mantle_crust_weight(theta_nu, pnu, flavors, cross_section_type='ctw'):
    """
    calculates neutrino weight due to Earth absorption with a three layers earth model, i.e. probability of the
    neutrino to reach the detector

    simple parametrization using momentum, zenith angle, flavor and current type information
    of the neutrino

    as implemented in ARAsim (2018)

    Parameters
    ----------
    theta_nu: float or array of floats
        the zenith angle of the neutrino direction (where it came from, i.e., opposite to the direction of propagation)
    pnu: float or array of floats
        the momentum of the neutrino
    flavors: float or array of floats
        the flavor of the neutrino
    """
    sigma = cross_sections.get_nu_cross_section(pnu, flavors, cross_section_type = cross_section_type)
    if(theta_nu <= 0.5 * np.pi):  # coming from above
        return np.ones_like(theta_nu)
    elif (theta_nu <= np.pi - np.arcsin(radii[1] / radii[2])): # only go through the outer layer
        d_outer = - 2 * R_EARTH * np.cos(theta_nu)
        weight = np.exp(-d_outer * sigma * densities[2] / AMU)
    elif (theta_nu <= np.pi - np.arcsin(radii[0] / radii[2])): # only go through the outer and middle layer
        d_middle = 2 * np.sqrt(radii[1] * radii[1] - radii[2] * radii[2] * np.sin(np.pi - theta_nu) * np.sin(np.pi - theta_nu))
        d_outer = - 2 * R_EARTH * np.cos(theta_nu) - d_middle
        weight = np.exp(-d_outer * sigma * densities[2] / AMU - d_middle * sigma * densities[1] / AMU)
    else: # go through all three layers
        d_inner = 2 * np.sqrt(radii[0] * radii[0] - radii[2] * radii[2] * np.sin(np.pi - theta_nu) * np.sin(np.pi - theta_nu))
        d_middle = 2 * np.sqrt(radii[1] * radii[1] - radii[2] * radii[2] * np.sin(np.pi - theta_nu) * np.sin(np.pi - theta_nu)) - d_inner
        d_outer = - 2 * R_EARTH * np.cos(theta_nu) - d_middle - d_inner
        weight = np.exp(-d_outer * sigma * densities[2] / AMU - d_middle * sigma * densities[1] / AMU - d_inner * sigma * densities[0] / AMU)
    return weight




from __future__ import absolute_import, division, print_function
import numpy as np
from radiotools import helper as hp
from radiotools import plthelpers as php
from matplotlib import pyplot as plt
from NuRadioMC.utilities import units
from NuRadioMC.utilities import medium
import h5py
import argparse
import json
import time
import os
from matplotlib.ticker import MultipleLocator, FormatStrFormatter


parser = argparse.ArgumentParser(description='Plot NuRadioMC event list output.')
parser.add_argument('inputfilename', type=str,
                    help='path to NuRadioMC hdf5 simulation output')
args = parser.parse_args()

fin = h5py.File(args.inputfilename, 'r')

weights = np.array(fin['weights'])
n_events = fin.attrs['n_events']
Vrms = fin.attrs['Vrms']

depths = np.array(fin['station_101'].attrs['antenna_positions'])[:, 2]
print("simulated depths ", depths)

eff1 = np.zeros_like(depths)
eff2 = np.zeros_like(depths)
eff3 = np.zeros_like(depths)

for iD, depth in enumerate(depths):
    As = np.array(fin['station_101']['max_amp_ray_solution'][:,iD])
    As = np.nan_to_num(As)  # this sets all cases with empty ray tracing solution to an amplitude of zero
    mask = np.isnan(As[:, 0]) | np.isnan(As[:, 1])
    As = As[~mask]
    Ts = np.array(fin['station_101']['travel_times'][:, iD])[~mask]
    Ts = np.nan_to_num(Ts)
    dTs = np.abs(Ts[:, 1] - Ts[:, 0])
    C3 = (As[:, 0] >= 3 * Vrms) | (As[:, 1] >= 3 * Vrms)
    
    C1 = ((As[:, 0] >= 3 * Vrms) | (As[:, 1] >= 3 * Vrms)) & ((As[:, 0] >= 2 * Vrms) & (As[:, 1] >= 2 * Vrms)) & (dTs < 430 * units.ns)
    C2 = ((As[:, 0] >= 3 * Vrms) & (As[:, 1] >= 3 * Vrms)) & (dTs < 430 * units.ns)
    C4 = ((As[:, 0] >= 4 * Vrms) & (As[:, 1] >= 4 * Vrms)) & (dTs < 430 * units.ns)
    
    eff1[iD] = np.sum(weights[~mask][C1])/ np.sum(weights[~mask][C3])
    eff2[iD] = np.sum(weights[~mask][C2])/ np.sum(weights[~mask][C3])
    eff3[iD] = np.sum(weights[~mask][C4])/ np.sum(weights[~mask][C3])
    
sort_mask = np.argsort(depths)
np.savetxt("DnRefficiency_{:.0g}eV.txt".format(np.array(fin['energies']).mean()), [depths[sort_mask], eff1[sort_mask]])

fig, (ax) = plt.subplots(1, 1, figsize=(6, 5))
ax.plot(depths[sort_mask], eff1[sort_mask], 'o-', label='one pulse > 3 sigma, one pulse > 2 sigma')
ax.plot(depths[sort_mask], eff2[sort_mask], "d--", label="both pulses > 3 sigma")
ax.plot(depths[sort_mask], eff3[sort_mask], "^:", label="both pulses > 4 sigma")
ax.set_xlim(0, -100)
ax.set_ylim(0, 1)
ax.set_xlabel("depth [m]")
ax.set_ylabel("efficiency")
ax.set_title("neutrino energy {:.2g}eV".format(np.array(fin['energies']).mean()))
ax.legend(fontsize='x-small', numpoints=1)
fig.tight_layout()
fig.savefig("{}.pdf".format(os.path.basename(args.inputfilename)))
plt.show()


from __future__ import absolute_import, division, print_function
import numpy as np
from NuRadioMC.utilities import units

# define simulation volume
xmin = -3 * units.km
xmax = 3 * units.km
ymin = -3 * units.km
ymax = 3 * units.km
fiducial_zmin = -3 * units.km
fiducial_zmax = 0 * units.km

HEADER = """VERSION=0.2
# standard event list format of NuRadioMC (adapted from ARASim)
# the very first line defines the file version, all other lines represent the events
# each row specifies one event
# all quantities are in the default NuRadioMC units (i.e., meters, radians and eV)
# all geometry quantities are in the NuRadioMC default local coordinate system:
#     coordinate origin is at the surface
#     x axis is towards Easting, y axis towards Northing, z axis upwards
#     zenith/theta angle is defined with respect to z axis, i.e. 0deg = upwards, 90deg = towards horizon, 180deg = downwards
#     azimuth/phi angle counting northwards from East
#
# the collumns are defined as follows
# 1. event id (integer)
# 2. neutrino flavor (integer) encoded as using PDF numbering scheme, particles have positive sign, anti-particles have negative sign, relevant for us are:
#       12: electron neutrino
#       14: muon neutrino
#       16: tau neutrino
# 3. energy of neutrino (double)
# 4. charge or neutral current interaction (string, one of ['cc', 'nc']
# 5./6./7. position of neutrino interaction vertex in cartesian coordinates (x, y, z) (in default NuRadioMC local coordinate system)
# 8. zenith/theta angle of neutrino direction (pointing into direction of propagation)
# 9. azimuth/phi angle of neutrino direction (pointing into direction of propagation)
# 10. inelasticity (the fraction of neutrino energy that goes into the hadronic part)
#
"""


def generate_eventlist(filename, n_events, Emin, Emax,
                       start_event_id=1,
                       flavor=[12, -12, 14, -14, 16, -16]):
    n_events = int(n_events)
    event_ids = np.arange(n_events) + start_event_id

    # generate neutrino flavors randomly
    flavors = np.array([flavor[i] for i in np.random.randint(0, high=len(flavor), size=n_events)])
    # generate energies randomly
    energies = 10 ** np.random.uniform(np.log10(Emin), np.log10(Emax), n_events)

    # generate charged/neutral current randomly (ported from ShelfMC)
    rnd = np.random.uniform(0., 1., n_events)
    ccncs = np.ones(n_events, dtype='S2')
    for i, r in enumerate(rnd):
        if(r <= 0.7064):
            ccncs[i] = 'cc'
        else:
            ccncs[i] = 'nc'

    # generate neutrino vertices randomly
    xx = np.random.uniform(xmin, xmax, n_events)
    yy = np.random.uniform(ymin, ymax, n_events)
    zz = np.random.uniform(fiducial_zmin, fiducial_zmax, n_events)

    # generate neutrino direction randomly
    azimuths = np.random.uniform(0, 360 * units.deg, n_events)
    u = np.random.uniform(-1, 1, n_events)
    zeniths = np.arccos(u)  # generates distribution that is uniform in cos(theta)

    # generate inelasticity (ported from ShelfMC)
    R1 = 0.36787944
    R2 = 0.63212056
    inelasticity = (-np.log(R1 + np.random.uniform(0., 1., n_events) * R2)) ** 2.5

    with open(filename, 'w') as fout:
        fout.write(HEADER)
        for i in range(n_events):
            fout.write("{:08d} {:>+5d}  {:.5e}  {:s}  {:>10.3f}  {:>10.3f}  {:>10.3f}  {:>10.3f}  {:>10.3f}  {:>10.3f}\n".format(event_ids[i], flavors[i], energies[i], ccncs[i], xx[i], yy[i], zz[i], zeniths[i], azimuths[i], inelasticity[i]))
        fout.close()


if __name__ == '__main__':
    generate_eventlist('test.txt', 1e5, 1e18, 1e18)




import NuRadioMC.SignalGen.RalstonBuniy.askaryan_module as AskaryanModule
import NuRadioMC.SignalGen.parametrizations as param
from NuRadioMC.utilities import units, fft
import numpy as np
from radiotools import plthelpers as php
import matplotlib.pyplot as plt

E = 1e17 * units.eV
n_index = 1.78
domega = 2.5 * units.deg
theta = np.arccos(1. / n_index) + domega

dt = .1 * units.ns
n_samples = 2048
ff = np.fft.rfftfreq(n_samples, dt)
df = ff[1]  # width of a frequency bin
tt = np.arange(0, n_samples * dt, dt)
R = 5 * units.km
em = True
a = 1.5 * units.m

fig, ax = plt.subplots(1, 1)
domegas = np.array([0, 2.5, 5, 7.5, 10]) * units.deg
for iOmega, domega in enumerate(domegas):
    theta = np.arccos(1. / n_index) + domega
    spec1 = AskaryanModule.get_frequency_spectrum(E, theta, ff, em, n_index, R, True, a)
    spec1_anoLPM = AskaryanModule.get_frequency_spectrum(E, theta, ff, em, n_index, R, False)
    spec1_aLPM = AskaryanModule.get_frequency_spectrum(E, theta, ff, em, n_index, R, True)
    spec4 = param.get_frequency_spectrum(E, theta, ff, em, n_index, R, model='ZHS1992')
    ax.plot(ff / units.MHz, np.abs(spec1[1]) / units.V * units.m, php.get_color(iOmega) + "-", label=r'JCH/AC a=1.5m $\Delta \Omega$ = {:.1f}deg'.format(domega / units.deg))
#     ax.plot(ff / units.MHz, np.abs(spec1_noLPM[1]) / units.V * units.m, php.get_color_linestyle(1), label='Askaryan module eTheta (without LPM)')
    ax.plot(ff / units.MHz, np.abs(spec4) / units.V * units.m, php.get_color(iOmega) + "--", label=r'ZHS $\Delta \Omega$ = {:.1f}deg'.format(domega / units.deg))
ax.semilogx(True)
ax.semilogy(True)
ax.set_xlim(10, 2e3)
ax.set_ylim(1e-9, 1e-3)
ax.set_xlabel("frequency [MHz]")
ax.set_ylabel("amplitude [V/m] per {:.1f}MHz".format(df / units.MHz))
ax.legend(fontsize='small')
ax.set_title("E = {:.1e}eV, R = {:.1f}km, EM={}".format(E / units.eV, R / units.km, em))
fig.tight_layout()
fig.savefig("plots/AskaryanZHS.png")

domegas = np.array([0, 1, 2.5, 5, 7.5, 10]) * units.deg
for iOmega, domega in enumerate(domegas):
    fig, ax = plt.subplots(1, 1)
    theta = np.arccos(1. / n_index) + domega
    spec1 = AskaryanModule.get_frequency_spectrum(E, theta, ff, em, n_index, R, True, a)
    spec1_anoLPM = AskaryanModule.get_frequency_spectrum(E, theta, ff, em, n_index, R, False)
    spec1_aLPM = AskaryanModule.get_frequency_spectrum(E, theta, ff, em, n_index, R, True)
    spec4 = param.get_frequency_spectrum(E, theta, ff, em, n_index, R, model='ZHS1992')
    ax.plot(ff / units.MHz, np.abs(spec1[1]) / units.V * units.m, php.get_color(0) + "-", label=r'JCH/AC a=1.5m')
#     ax.plot(ff / units.MHz, np.abs(spec1_noLPM[1]) / units.V * units.m, php.get_color_linestyle(1), label='Askaryan module eTheta (without LPM)')
    ax.plot(ff / units.MHz, np.abs(spec4) / units.V * units.m, php.get_color(1) + "--", label=r'ZHS')
    ax.plot(ff / units.MHz, np.abs(spec1_anoLPM[1]) / units.V * units.m, php.get_color(2) + ":", label=r'JCH/AC without LPM')
    ax.plot(ff / units.MHz, np.abs(spec1_aLPM[1]) / units.V * units.m, php.get_color(3) + "-.", label='JCH/AC with LPM')
    ax.semilogx(True)
    ax.semilogy(True)
    ax.set_xlim(10, 2e3)
    ax.set_ylim(1e-9, 1e-3)
    ax.set_xlabel("frequency [MHz]")
    ax.set_ylabel("amplitude [V/m] per {:.1f}MHz".format(df / units.MHz))
    ax.legend(fontsize='small')
    ax.set_title("E = {:.1e}eV, $\Delta\Omega$ = {:.1f}deg, R = {:.1f}km, EM={}".format(E / units.eV, domega / units.deg, R / units.km, em))
    fig.tight_layout()
    fig.savefig("plots/AskaryanZHS_{:.1f}deg.png".format(domega / units.deg))
    plt.close('all')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import numpy as np
from NuRadioReco.utilities import units
from scipy import interpolate as intp
from scipy import integrate as int
from scipy import constants
from matplotlib import pyplot as plt
import os
import pickle
from time import time
from NuRadioMC.SignalGen.ARZ import ARZ
from radiotools import coordinatesystems as cstrafo
import logging
logging.basicConfig(level=logging.INFO)

shower_energy = 1.24e18 *units.eV
theta = 56 * units.deg
R = 50000 * units.km
N = 512
dt = 0.1 * units.ns
n_index = 1.78
shower_type = "EM"
tt = np.arange(0, dt*N, dt)

cARZ = ARZ.ARZ()

cARZ.set_seed(100)
trace = cARZ.get_time_trace(shower_energy, theta, N, dt, shower_type, n_index, R, shift_for_xmax=False)
fig, ax = plt.subplots(1, 1)
ax.plot(tt, trace[1]/units.V * units.m / units.milli, label='eTheta')
ax.plot(tt, trace[2]/units.V * units.m / units.milli, label='ePhi')
ax.plot(tt, trace[0]/units.V * units.m / units.milli, label='eR')
ax.set_xlabel("time [ns]")
ax.set_ylabel("amplitude [mV/m]")
ax.legend()
fig.tight_layout()

cARZ.set_seed(100)
trace = cARZ.get_time_trace(shower_energy, theta, N, dt, shower_type, n_index, R, shift_for_xmax=False)
fig, ax = plt.subplots(1, 1)
ax.plot(tt, trace[1]/units.V * units.m / units.milli, label='eTheta')
ax.plot(tt, trace[2]/units.V * units.m / units.milli, label='ePhi')
ax.plot(tt, trace[0]/units.V * units.m / units.milli, label='eR')
ax.set_xlabel("time [ns]")
ax.set_ylabel("amplitude [mV/m]")
ax.set_title("2")
ax.legend()
fig.tight_layout()

cARZ.set_seed(100)
trace2 = cARZ.get_time_trace(shower_energy, theta, N, dt, shower_type, n_index, R)

fig, ax = plt.subplots(1, 1)
ax.plot(tt, trace2[1]/units.V * units.m / units.milli, label='eTheta')
ax.plot(tt, trace2[2]/units.V * units.m / units.milli, label='ePhi')
ax.plot(tt, trace2[0]/units.V * units.m / units.milli, label='eR')
ax.set_title("shifted for xmax")
ax.set_xlabel("time [ns]")
ax.set_ylabel("amplitude [mV/m]")
ax.legend()
fig.tight_layout()

plt.show()



from NuRadioMC.EvtGen.generator import *
import matplotlib.pyplot as plt

tau_energy = 1.e19 * units.eV
branch = 'tau_pi'

def f_1(y,r):
    return -(2*y+1+r)*(1-2*r)/(1-r)**2/(1+2*r)
def f_0(y,r):
    return 1/(1-r)
def f(y,r):
    if ( y < 0 or y > 1-r**2 ):
        return 0
    else:
        return f_1(y,r)+f_0(y,r)

def f_pi(y,r):
    if ( y < 0 or y > 1-r**2 ):
        return 0
    else:
        return f_0(y,r)-(2*y-1+r)/(1-r**2)**2

r = rho770_mass/tau_mass
print(r)
x = np.linspace(0,1,100)
distr = [f(x0,r) for x0 in x]
plt.plot(x, distr)
plt.show()

energies = []
for i in range(10000):
    products = products_from_tau_decay(tau_energy, branch)
    energies.append(products[16])

plt.hist(energies)
plt.show()



import glob
import os

# define the base directory of this job 
base_dir ="/pub/arianna/NuRadioMC/station_designs"
# specify the NuRadioMC python steering file (needs to be in base_dir)  
detector_sim = 'T02RunSimulation.py'
# specify detector description (needs to be in base_dir)
detector_filename = 'surface_station_1GHz.json'
# specify directory that contains the detector descriptions
det_dir =  os.path.join(base_dir, "detectors")
# specify the NuRadioMC config file that should be used for this simulation
config_file =  os.path.join(base_dir, "config_5.yaml")
# specify a working directory for this specific simulation run
working_dir =  os.path.join(base_dir, "suface_station_1GHz/05")
# specify the directory containing the input event files, this directory needs to contain separate folders
input_dir = "/pub/arianna/NuRadioMC/input_8km_1e7_1e5/"
# specify the path to the software directory (where NuRadioMC, NuRadioReco and radiotools are installed in)
software = '/data/users/jcglaser/software'

# run and output directories are created automatically if not yet present
if not os.path.exists(os.path.join(working_dir, "output")):
    os.makedirs(os.path.join(working_dir, "output"))
if not os.path.exists(os.path.join(working_dir, "run")):
    os.makedirs(os.path.join(working_dir, "run"))

# loop over all input event files and create a job script for each input file. 
for iF, filename in enumerate(sorted(glob.glob(os.path.join(input_dir, '*/*.hdf5.*')))):
    current_folder = os.path.split(os.path.dirname(filename))[-1]
    detector_file = os.path.join(det_dir, detector_filename)
    # check if subfolder for energies exist
    t1 = os.path.join(working_dir, "output", current_folder)
    if(not os.path.exists(t1)):
        os.makedirs(t1)
    t1 = os.path.join(working_dir, 'run', current_folder)
    if(not os.path.exists(t1)):
        os.makedirs(t1)
    output_filename = os.path.join(working_dir, "output", current_folder, os.path.basename(filename))
    cmd = "python {} {} {} {} {}\n".format(os.path.join(base_dir, detector_sim), filename, detector_file, config_file,
                                           output_filename)

    # here we add specific settings for the grid engine job scheduler, this part need to be adjusted to the specifics 
    # of your cluster
    header = '#!/bin/bash\n'
    header += '#$ -N C_{}\n'.format(iF)
    header += '#$ -j y\n'
    header += '#$ -V\n'
    header += '#$ -q grb,grb64\n'
    header += '#$ -ckpt restart\n'  # restart jobs in case of a node crash
    header += '#$ -o {}\n'.format(os.path.join(working_dir, 'run'))
    
    # add the software to the PYTHONPATH
    header += 'export PYTHONPATH={}/NuRadioMC:$PYTHONPATH\n'.format(software)
    header += 'export PYTHONPATH={}/NuRadioReco:$PYTHONPATH \n'.format(software)
    header += 'export PYTHONPATH={}/radiotools:$PYTHONPATH \n'.format(software)
    header += 'cd {} \n'.format(working_dir)

    with open(os.path.join(working_dir, 'run', current_folder, os.path.basename(filename) + ".sh"), 'w') as fout:
        fout.write(header)
        fout.write(cmd)



import json
import numpy as np
import matplotlib.pyplot as plt

with open('interp_1e-3.json', 'r') as f:
    interp = json.load(f)

colours = ['red', 'black', 'blue', 'orange', 'cadetblue']
colour_dict = {}

for iR, R in enumerate(interp):

    colour_dict[R] = colours[iR]

for R in interp:

    thetas = interp[R].keys().sort()
    thetas = [ float(theta) for theta in interp[R].keys() ]
    thetas = np.sort(np.array(thetas))
    thetas_plot = np.array(thetas)

    interp_factors = [ interp[R][theta] for theta in interp[R].keys() ]
    interp_factors = np.array(interp_factors)

    plt.subplot(2,1,1)
    plt.plot(thetas_plot, interp_factors[:,0], label=R+' m, interp 1',
            color = colour_dict[R], linestyle = '-', marker='o')
    plt.subplot(2,1,2)
    plt.plot(thetas_plot, interp_factors[:,1], label=R+' m, interp 2',
            color = colour_dict[R], linestyle = '--', marker='o')

plt.subplot(2,1,1)
plt.ylabel('Interp factor 1')
plt.legend()
plt.subplot(2,1,2)
plt.ylabel('Interp factor 2')
plt.xlabel('Angle [deg]')
plt.savefig('interpolation_plot.png', format='png')
plt.show()



from __future__ import absolute_import, division, print_function, unicode_literals
import matplotlib.pyplot as plt
import logging
logging.basicConfig(level=logging.WARNING)
import numpy as np
from scipy import signal
import argparse
from datetime import datetime
import pickle
from NuRadioReco.utilities import units
from radiotools import plthelpers as php

i = 2

with open("sim_results_{:02d}.pkl".format(i), 'r') as fin:
    results = pickle.load(fin)
    d = results['depth']
    zen = np.array(results['exp'])[:, 0]
    az = np.array(results['exp'])[:, 1]
    
    fig, (ax, ax2) = plt.subplots(2, 1, sharex=True)
    ax.scatter(d, (np.array(results['corr_LPDA'])[:, 0] - zen) / units.deg, label='corr LPDA',
               s=10, marker='o', alpha=0.5)
    ax.scatter(d, (np.array(results['corr_dipole'])[:, 0] - zen) / units.deg, label='corr dipole',
               s=10, marker='d', alpha=0.5)
#     ax.scatter(d, (np.array(results['time_LPDA'])[:, 0] - zen) / units.deg, label='times LPDA',
#                s=10, marker='^', alpha=0.5)
#     ax.scatter(d, (np.array(results['time_dipole'])[:, 0] - zen) / units.deg, label='times dipole',
#                s=10, marker='x', alpha=0.5)
    ax.set_ylabel("zenith (reco - exp) [deg]")
    ax.legend()
    
    ax2.scatter(d, (np.array(results['corr_LPDA'])[:, 1] - az) / units.deg, label='corr LPDA',
                s=10, marker='o', alpha=0.5)
    ax2.scatter(d, (np.array(results['corr_dipole'])[:, 1] - az) / units.deg, label='corr dipole',
                s=10, marker='d', alpha=0.5)
#     ax2.scatter(d, (np.array(results['time_LPDA'])[:, 1] - az) / units.deg, label='times LPDA',
#                 s=10, marker='^', alpha=0.5)
#     ax2.scatter(d, (np.array(results['time_dipole'])[:, 1] - az) / units.deg, label='times dipole',
#                 s=10, marker='x', alpha=0.5)
    ax2.set_ylabel("azimuth (reco - exp) [deg]")
    ax2.set_xlabel("depth [m]")
    fig.tight_layout()
    
    
    
    fig, ax = php.get_histograms([(np.array(results['corr_LPDA'])[:, 0] - zen) / units.deg,
                        (np.array(results['corr_dipole'])[:, 0] - zen) / units.deg,
                        (np.array(results['time_LPDA'])[:, 0] - zen) / units.deg,
                        (np.array(results['time_dipole'])[:, 0] - zen) / units.deg,
                        (np.array(results['corr_LPDA'])[:, 1] - az) / units.deg,
                        (np.array(results['corr_dipole'])[:, 1] - az) / units.deg,
                        (np.array(results['time_LPDA'])[:, 1] - az) / units.deg,
                        (np.array(results['time_dipole'])[:, 1] - az) / units.deg],
                        bins=np.linspace(-15, 15, 30),
                        titles=['corr LPDA', 'corr_dipole', 'time_LPDA', 'time_dipole', '','','',''],
                        xlabels=[r'$\Delta$zen [deg]', r'$\Delta$zen [deg]', r'$\Delta$zen [deg]', r'$\Delta$zen [deg]',
                                 r'$\Delta$az [deg]', r'$\Delta$az [deg]', r'$\Delta$az [deg]', r'$\Delta$az [deg]'])
    fig.savefig("plots/results_{:02d}_hist.png".format(i))
    plt.show()



import numpy as np
import h5py
from scipy import interpolate
import glob
from NuRadioMC.utilities import units
import json
import os


# collection of utility function regarding the calculation of the effective volume of a neutrino detector

def get_triggered(fin):
    """
    Computes an array indicating the triggered events.
    If a double bang is seen, removes the second bang from the actual triggered
    array so as not to count twice the same event for the effective volume.

    Parameters
    ----------
    fin: dictionary
       Dictionary containing the output data sets from the simulation

    Returns
    -------
    triggered: numpy array with bools
       The bools indicate if the events have triggered
    """

    triggered = fin['triggered'][:]

    if (len(triggered) == 0):
        return triggered

    second_bang_indexes = np.argwhere(np.array(fin['n_interaction']) >= 2)
    if(second_bang_indexes.shape[0] > 1):
        second_bang_indexes = np.squeeze(second_bang_indexes)
    elif(second_bang_indexes.shape[0] == 1):
        second_bang_indexes = second_bang_indexes[0]
    else:
        return triggered

    for second_bang_index in second_bang_indexes:
        # Second interactions
        if (fin['event_ids'][second_bang_index-1] == fin['event_ids'][second_bang_index]
            and fin['triggered'][second_bang_index-1] == True
            and fin['triggered'][second_bang_index] == True):

            triggered[second_bang_index] = False

        # Third interactions
        elif (fin['event_ids'][second_bang_index-2] == fin['event_ids'][second_bang_index]
              and fin['triggered'][second_bang_index-2] == True
              and fin['triggered'][second_bang_index] == True):

            triggered[second_bang_index] = False

    return triggered

def get_Veff(folder, trigger_combinations={}, zenithbins=False):
    """
    calculates the effective volume from NuRadioMC hdf5 files

    Parameters
    ----------
    folder: string
        folder conaining the hdf5 files, one per energy
    trigger_combinations: dict, optional
        keys are the names of triggers to calculate. Values are dicts again:
            * 'triggers': list of strings
                name of individual triggers that are combines with an OR
            * 'efficiency': string
                the signal efficiency vs. SNR (=Vmax/Vrms) to use. E.g. 'Chris'
            * 'efficiency_scale': float
                rescaling of the efficiency curve by SNR' = SNR * scale
    zenithbins: bool
        If true, returns the minimum and maximum zenith angles

    Returns
    ----------
    np.array(Es): numpy floats array
        Smallest energy for each bin
    Veffs: floats list
        Effective volumes
    Veffs_error: floats list
        Effective volume uncertainties
    SNR: floats list
        Signal to noise ratios
    trigger_names: string list
        Trigger names
    [thetamin, thetamax]: [float, float]
        Mimimum and maximum zenith angles
    deposited: bool
        True if the energies are deposited energies
        False if the energies are primary neutrino energies
    """
    trigger_names = None
    trigger_names_dict = {}
    Veffs = {}
    SNR = {}
    Veffs_error = {}
    Es = []
    prev_deposited = None
    deposited = False

    for iF, filename in enumerate(sorted(glob.glob(os.path.join(folder, '*.hdf5')))):
        fin = h5py.File(filename, 'r')
        if 'deposited' in fin.attrs:
            deposited = fin.attrs['deposited']
            if prev_deposited is None:
                prev_deposited = deposited
            elif prev_deposited != deposited:
                print("Warning! The deposited parameter is not consistent!")

        if('trigger_names' in fin.attrs):
            trigger_names = fin.attrs['trigger_names']
        if(len(trigger_names) > 0):
            for iT, trigger_name in enumerate(trigger_names):
                Veffs[trigger_name] = []
                Veffs_error[trigger_name] = []
                trigger_names_dict[trigger_name] = iT
            break

    trigger_combinations['all_triggers'] = {'triggers': trigger_names}
    print("Trigger names:", trigger_names)

    for iF, filename in enumerate(sorted(glob.glob(os.path.join(folder, '*.hdf5')))):
        print(filename)
        fin = h5py.File(filename, 'r')
        E = fin.attrs['Emin']
        Es.append(E)

        weights = np.array(fin['weights'])
        #triggered = np.array(fin['triggered'])
        triggered = get_triggered(fin)
        n_events = fin.attrs['n_events']
        if(trigger_names is None):
            trigger_names = fin.attrs['trigger_names']
            for iT, trigger_name in enumerate(trigger_names):
                Veffs[trigger_name] = []
                Veffs_error[trigger_name] = []
                trigger_names_dict[trigger_name] = iT
            print(trigger_names)
        else:
            if(np.any(trigger_names != fin.attrs['trigger_names'])):

                if( triggered.size == 0 and fin.attrs['trigger_names'].size == 0 ):
                    print("file {} has not triggering events. Using trigger names from another file".format(filename))
                else:
                    print("file {} has inconsistent trigger names: {}".format(filename, fin.attrs['trigger_names']))
                    raise

        # calculate effective
        density_ice = 0.9167 * units.g / units.cm ** 3
        density_water = 997 * units.kg / units.m ** 3
        rmin = fin.attrs['rmin']
        rmax = fin.attrs['rmax']
        thetamin = 0 
        thetamax = np.pi
        phimin = 0
        phimax = 2 * np.pi
        if('thetamin' in fin.attrs):
            thetamin = fin.attrs['thetamin']
        if('thetamax' in fin.attrs):
            thetamax = fin.attrs['thetamax']
        if('phimin' in fin.attrs):
            fin.attrs['phimin']
        if('phimax' in fin.attrs):
            fin.attrs['phimax']
        dZ = fin.attrs['zmax'] - fin.attrs['zmin']
        V = np.pi * (rmax**2 - rmin**2) * dZ
        Vrms = fin.attrs['Vrms']

        # Solid angle needed for the effective volume calculations
        omega = np.abs(phimax - phimin) * np.abs( np.cos(thetamin)-np.cos(thetamax) )

        for iT, trigger_name in enumerate(trigger_names):
            triggered = np.array(fin['multiple_triggers'][:, iT], dtype=np.bool)
            Veff = V * density_ice / density_water * omega * np.sum(weights[triggered]) / n_events
            Veffs[trigger_name].append(Veff)
            try:
                Veffs_error[trigger_name].append(Veff / np.sum(weights[triggered])**0.5)
            except:
                Veffs_error[trigger_name].append(np.nan)
#             print("{}: log(E) = {:.3g}, Veff = {:.3f}km^3 st".format(trigger_name, np.log10(E), Veff / units.km**3))

        for trigger_name, values in trigger_combinations.iteritems():
            indiv_triggers = values['triggers']
            if(trigger_name not in Veffs):
                Veffs[trigger_name] = []
                Veffs_error[trigger_name] = []
            triggered = np.zeros_like(fin['multiple_triggers'][:, 0], dtype=np.bool)
            if(isinstance(indiv_triggers, str)):
                triggered = triggered | np.array(fin['multiple_triggers'][:, trigger_names_dict[indiv_triggers]], dtype=np.bool)
            else:
                for indiv_trigger in indiv_triggers:
                    triggered = triggered | np.array(fin['multiple_triggers'][:, trigger_names_dict[indiv_trigger]], dtype=np.bool)
            if 'triggerAND' in values:
                triggered = triggered & np.array(fin['multiple_triggers'][:, trigger_names_dict[values['triggerAND']]], dtype=np.bool)
            if 'notriggers' in values:
                indiv_triggers = values['notriggers']
                if(isinstance(indiv_triggers, str)):
                    triggered = triggered & ~np.array(fin['multiple_triggers'][:, trigger_names_dict[indiv_triggers]], dtype=np.bool)
                else:
                    for indiv_trigger in indiv_triggers:
                        triggered = triggered & ~np.array(fin['multiple_triggers'][:, trigger_names_dict[indiv_trigger]], dtype=np.bool)
            if('min_sigma' in values.keys()):
                if(isinstance(values['min_sigma'], list)):
                    if(trigger_name not in SNR):
                        SNR[trigger_name] = {}
                    masks = np.zeros_like(triggered)
                    for iS in range(len(values['min_sigma'])):
#                         As = np.array(fin['maximum_amplitudes'])
                        As = np.max(np.nan_to_num(fin['max_amp_ray_solution']), axis=-1) # we use the this quantity because it is always computed before noise is added!
                        As_sorted = np.sort(As[:, values['channels'][iS]], axis=1)
                        # the smallest of the three largest amplitudes
                        max_amplitude = As_sorted[:, -values['n_channels'][iS]]
                        mask = np.sum(As[:, values['channels'][iS]] >= (values['min_sigma'][iS] * Vrms), axis=1) >= values['n_channels'][iS]
                        masks = masks | mask
                        if(iS not in SNR[trigger_name]):
                            SNR[trigger_name][iS] = []
                        SNR[trigger_name][iS].append([max_amplitude[mask] / Vrms])
                    triggered = triggered & masks
                else:
                    if(trigger_name not in SNR):
                        SNR[trigger_name] = []
#                     As = np.array(fin['maximum_amplitudes'])
                    As = np.max(np.nan_to_num(fin['max_amp_ray_solution']), axis=-1) # we use the this quantity because it is always computed before noise is added!
    #                 print(As.shape)
#                     print(trigger_name, values['channels'])
                    As_sorted = np.sort(As[:, values['channels']], axis=1)
                    max_amplitude = As_sorted[:, -values['n_channels']]  # the smallest of the three largest amplitudes
    #                 print(np.sum(As_sorted[:, -values['n_channels']] > (values['min_sigma'] * Vrms)))
                    mask = np.sum(As[:, values['channels']] >= (values['min_sigma'] * Vrms), axis=1) >= values['n_channels']
    #                 max_amplitude[~mask] = 0
                    SNR[trigger_name].append(As_sorted[mask] / Vrms)
    #                 print(Vrms, mask.shape, np.sum(mask))
                    triggered = triggered & mask
            if('ray_solution' in values.keys()):
                As = np.array(fin['max_amp_ray_solution'])
                max_amps = np.argmax(As[:, values['ray_channel']], axis=-1)
                sol = np.array(fin['ray_tracing_solution_type'])
#                 print(sol[:,values['ray_channel']][max_amps].shape)
#                 print(max_amps.shape)
#                 a = 1/0
                mask = np.array([sol[i,values['ray_channel'], max_amps[i]] == values['ray_solution'] for i in range(len(max_amps))], dtype=np.bool)
                triggered = triggered & mask

            Veff = V * density_ice / density_water * omega * np.sum(weights[triggered]) / n_events

            if('efficiency' in values.keys()):
                SNReff, eff = np.loadtxt("analysis_efficiency_{}.csv".format(values['efficiency']), delimiter=",", unpack=True)
                get_eff = interpolate.interp1d(SNReff, eff, bounds_error=False, fill_value=(0, eff[-1]))
                As = np.max(np.max(np.nan_to_num(fin['max_amp_ray_solution']), axis=-1)[:,np.append(range(0, 8), range(12, 20))], axis=-1) # we use the this quantity because it is always computed before noise is added!
                if('efficiency_scale' in values.keys()):
                    As *= values['efficiency_scale']
                e = get_eff(As/Vrms)
                Veff = V * density_ice / density_water * omega * np.sum((weights*e)[triggered]) / n_events

            Veffs[trigger_name].append(Veff)
            Veffs_error[trigger_name].append(Veff / np.sum(weights[triggered])**0.5)
    for trigger_name in Veffs.keys():
        Veffs[trigger_name] = np.array(Veffs[trigger_name])
        Veffs_error[trigger_name] = np.array(Veffs_error[trigger_name])

    if zenithbins:
        return np.array(Es), Veffs, Veffs_error, SNR, trigger_names, [thetamin, thetamax], deposited
    else:
        return np.array(Es), Veffs, Veffs_error, SNR, trigger_names, deposited

def exportVeff(filename, trigger_names, Es, Veffs, Veffs_error):
    """
    export effective volumes into a human readable JSON file

    Parameters
    ----------
    filename: string
        the output filename of the JSON file
    trigger_names: list of strings
        the triggers for which the effective volume is exported
    Es: list or array of floats
        the energies
    Veffs: dictionary
        dictionary containing Veffs for each trigger
    Veffs_error: dictionary
        dictionary containing Veff_errors for each trigger


    """
    output = {}
    for trigger_name in trigger_names:
        output[trigger_name] = {}
        output[trigger_name]['energies'] = list(Es)
        output[trigger_name]['Veff'] = list(Veffs[trigger_name])
        output[trigger_name]['Veff_error'] = list(Veffs_error[trigger_name])

    with open(filename, 'w') as fout:
        json.dump(output, fout, sort_keys=True, indent=4)

def exportVeffPerZenith(folderlist, outputfile):
    """
    export effective volumes into a human readable JSON file
    We assume a binning in zenithal angles

    Parameters
    ----------
    folderlist: strings list
        list containing the input folders
    outputfile: string
        name for the output file
    """
    output = {}
    for folder in folderlist:

        Es, Veffs, Veffs_error, SNR, trigger_names, thetas, deposited = get_Veff(folder, zenithbins=True)
        output[thetas[0]] = {}

        for trigger_name in trigger_names:
            output[thetas[0]][trigger_name] = {}
            if deposited:
                output[thetas[0]][trigger_name]['deposited_energies'] = list(Es)
            else:
                output[thetas[0]][trigger_name]['energies'] = list(Es)
            output[thetas[0]][trigger_name]['Veff'] = list(Veffs[trigger_name])
            output[thetas[0]][trigger_name]['Veff_error'] = list(Veffs_error[trigger_name])

    with open(outputfile, 'w+') as fout:

        json.dump(output, fout, sort_keys=True, indent=4)



import NuRadioMC.SignalGen.RalstonBuniy.askaryan_module as AskaryanModule
import NuRadioMC.SignalGen.parametrizations as param
from NuRadioMC.utilities import units, fft
import numpy as np
import matplotlib.pyplot as plt

E = 1e17 * units.eV
n_index = 1.78
domega = 2.5 * units.deg
theta = np.arccos(1. / n_index) + domega

dt = .1 * units.ns
n_samples = 2048
ff = np.fft.rfftfreq(n_samples, dt)
tt = np.arange(0, n_samples * dt, dt)
R = 5 * units.km
em = False

df = ff[1]  # width of a frequency bin

spec1 = AskaryanModule.get_frequency_spectrum(E, theta, ff, em, n_index, R, True)
spec1_noLPM = AskaryanModule.get_frequency_spectrum(E, theta, ff, em, n_index, R, False)
spec2 = param.get_frequency_spectrum(E, theta, ff, em, n_index, R, model='Alvarez2012')
spec3 = param.get_frequency_spectrum(E, theta, ff, em, n_index, R, model='Alvarez2000')
spec4 = param.get_frequency_spectrum(E, theta, ff, em, n_index, R, model='ZHS1992')

fig, ax = plt.subplots(1, 1)
# ax.plot(ff / units.MHz, np.abs(spec1[0]) / units.V * units.m, 'o', label='Askaryan module eR (without LPM)')
ax.plot(ff / units.MHz, np.abs(spec1[1]) / units.V * units.m, 'o', label='Askaryan module eTheta (with LPM)')
ax.plot(ff / units.MHz, np.abs(spec1_noLPM[1]) / units.V * units.m, 'o', label='Askaryan module eTheta (without LPM)')
ax.plot(ff / units.MHz, np.abs(spec2) / units.V * units.m, 'd', label='pyrex Alvarez2012')
ax.plot(ff / units.MHz, np.abs(spec3) / units.V * units.m, 's', label='shelfmc Alvarez2000')
ax.plot(ff / units.MHz, np.abs(spec4) / units.V * units.m, '>', label='ZHS1992')
ax.semilogx(True)
ax.semilogy(True)
ax.set_xlim(10, 2e3)
ax.set_ylim(1e-9, 1e-3)
ax.set_xlabel("frequency [MHz]")
ax.set_ylabel("amplitude [V/m] per {:.1f}MHz".format(df / units.MHz))
ax.legend(fontsize='small')
ax.set_title("E = {:.1e}eV, $\Delta\Omega$ = {:.1f}deg, R = {:.1f}km, EM={}".format(E/units.eV, domega / units.deg, R / units.km, em))
fig.tight_layout()
if(em):
    fig.savefig("comparison_em.png")
else:
    fig.savefig("comparison_had.png")
plt.show()

# test if different models are normalized such that they give the same time trace
# independent of the bin width

# 1st askaryan module
fig, ax = plt.subplots(1, 1)
n_trace = 2 ** 10  # samples of trace
dt = 0.1 * units.ns
tt = np.arange(0, n_trace * dt, dt)
ff = np.fft.rfftfreq(n_trace, dt)

trace = AskaryanModule.get_time_trace(E, theta, ff, 0, n=1.78, R=R)
ax.plot(tt / units.ns, np.abs(trace[0]), label="eR dt = {:.2f}ns, n={}".format(dt / units.ns, n_trace))
ax.plot(tt / units.ns, np.abs(trace[1]), label="dt = {:.2f}ns, n={}".format(dt / units.ns, n_trace))
print('maximum amplitude for {:d} samples is: {:4g}V/m'.format(n_trace, np.abs(trace).max()))

n_trace = 2 ** 12  # samples of trace
dt = 0.1 * units.ns
tt = np.arange(0, n_trace * dt, dt)
ff = np.fft.rfftfreq(n_trace, dt)
trace = AskaryanModule.get_time_trace(E, theta, ff, 0, n=1.78, R=R)
ax.plot(tt / units.ns, np.abs(trace[0]), label="eR dt = {:.2f}ns, n={}".format(dt / units.ns, n_trace))
ax.plot(tt / units.ns, np.abs(trace[1]), label="dt = {:.2f}ns, n={}".format(dt / units.ns, n_trace))
print('maximum amplitude for {:d} samples is: {:4g}V/m'.format(n_trace, np.abs(trace).max()))
ax.legend()
ax.set_title("askaryan module")
plt.show()
a = 1 / 0

# 2nd pyrex
fig, ax = plt.subplots(1, 1)
n_trace = 2 ** 10  # samples of trace
dt = 0.1 * units.ns
tt = np.arange(0, n_trace * dt, dt)
ff = np.fft.rfftfreq(n_trace, dt)

trace = fft.freq2time(param.get_frequency_spectrum(E, theta, ff, 0, n_index, R, model='Alvarez2012'))
ax.plot(tt / units.ns, trace, label="dt = {:.2f}ns, n={}".format(dt / units.ns, n_trace))

n_trace = 2 ** 12  # samples of trace
dt = 0.1 * units.ns
tt = np.arange(0, n_trace * dt, dt)
ff = np.fft.rfftfreq(n_trace, dt)
trace = fft.freq2time(param.get_frequency_spectrum(E, theta, ff, 0, n_index, R, model='Alvarez2012'))
ax.plot(tt / units.ns, trace, '--', label="dt = {:.2f}ns, n={}".format(dt / units.ns, n_trace))
ax.legend()
ax.set_title("pyrex")
plt.show()



"""
Messenger for HDF5 Manipulator
"""

VIOLET = '\033[95m'
BLUE = '\033[94m'
GREEN = '\033[92m'
YELLOW = '\033[93m'
RED = '\033[91m'
BOLD = '\033[1m'
UNDERLINE = '\033[4m'
END = '\033[0m'


def _print(text, indent, color):

    """Print text in given color, preceded by given #tabs.

    Keyword arguments:
    text -- text to print
    indent -- no. of tabs
    color -- text color defined by global variables
    """

    if indent:
        print '\t' * indent,
    print color + text + END


def info(text, indent=None):

    """Precede text with 'INFO' and call _print with GREEN color."""

    _print("\nINFO: " + text, indent, GREEN)


def warning(text, indent=None):

    """Precede text with 'WARNING' and call _print with YELLOW color."""

    _print("\nWARNING: " + text, indent, YELLOW)


def error(text, indent=None):

    """Precede text with 'ERROR' and call _print with RED color."""

    _print("\nERROR: " + text, indent, RED)


def box(text, width=80):

    """'Draw box' and print text in the center (using BOLD font)."""

    print BLUE
    pad = (width - len(text)) / 2
    print '+' + '-' * width + '+'
    print '|' + ' ' * width + '|'
    print '|' + ' ' * pad + text + ' ' * (width - pad - len(text)) + '|'
    print '|' + ' ' * width + '|'
    print '+' + '-' * width + '+'
    print END


def list_dataset(data, indent=1):

    """Print the list of datasets.

    Keyword arguments:
    data -- dictionary with data
    """

    adjust = len(max(data.keys(), key=len)) + 1  # length of left text column

    for key in data:
        print '\t' * indent + " - %(key)s %(type)s %(size)s" \
            % {"key": (key+':').ljust(adjust),
               "size": '-> ' + str(data[key].shape),
               "type": ('[' + str(data[key].dtype) + ']').ljust(9),
               }


def list_fileinfo(filename, range):

    """Print information about file to be saved.

    Keyword arguments:
    filename -- path to output hdf5 file
    range -- subset to be saved
    """

    print "\t - %(file)s: %(n)d entries from %(b)d to %(e)d" \
          % {"file": filename, "n": range[1] - range[0],
             "b": range[0], "e": range[1] - 1}
